package db

import (
	"applet/app/db/model"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"fmt"
	"github.com/syyongx/php2go"
	"strings"
	"xorm.io/xorm"
)

//统计金额
func GetCouponUserOrdBySum(Db *xorm.Engine, condition *model.CommCouponUserOrd, cols string) (float64, error) {
	total, err := Db.Sum(condition, cols)
	if err != nil {
		return 0, err
	}
	return total, err
}

//统计数量
func GetCouponUserOrdByCount(Db *xorm.Engine, condition *model.CommCouponUserOrd, createAt string) (total int64, err error) {
	if createAt != "" {
		total, err = Db.Where("create_at >= ? ", createAt).Count(condition)
	} else {
		total, err = Db.Count(condition)
	}
	fmt.Println(createAt)
	fmt.Println(total)
	if err != nil {
		return 0, err
	}
	return total, err
}

//统计金额
func GetCouponUserOrdBySumWithOrdType(Db *xorm.Engine, uid int, cols string, ordType ...int) (float64, error) {
	session := Db.Where("1=1")
	if uid > 0 {
		session = session.And("uid=?", uid)
	}
	if len(ordType) > 0 {
		session = session.In("ord_type", ordType)
	}
	total, err := session.Sum(&model.CommCouponUserOrd{}, cols)
	if err != nil {
		return 0, err
	}
	return total, err
}
func CouponUserOrdByInsert(session *xorm.Session, m *model.CommCouponUserOrd) bool {
	_, err := session.InsertOne(m)
	if err != nil {
		logx.Warn(err)
		return false
	}
	return true
}
func CouponUserOrdByInsertOne(eg *xorm.Engine, m *model.CommCouponUserOrd) bool {
	_, err := eg.InsertOne(m)
	if err != nil {
		logx.Warn(err)
		return false
	}
	return true
}
func CouponUserOrdByUpdate(eg *xorm.Engine, m *model.CommCouponUserOrd, cols string) bool {
	_, err := eg.Where("id=?", m.Id).Cols(cols).Update(m)
	if err != nil {
		logx.Warn(err)
		return false
	}
	return true
}
func commWhere(Db *xorm.Engine, uid int, types, pvdType, times string) *xorm.Session {
	var dbWhere = Db.Where("uid=? AND pvd_type=? AND create_at like ? ", uid, pvdType, times+"%")
	if types == "output" {
		dbWhere.And("type=?", 1)
	} else if types == "input" {
		dbWhere.And("type=?", 0)
	}
	return dbWhere
}
func CouponUserOrdList(Db *xorm.Engine, uid int, types, pvdType, times string, limit, start int) ([]*model.CommCouponUserOrd, error) {
	var m []*model.CommCouponUserOrd
	dbWhere := commWhere(Db, uid, types, pvdType, times)
	err := dbWhere.Desc("create_at").Limit(limit, start).Find(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	return m, nil
}
func CouponUserOrdByGid(session *xorm.Session, uid int, types, gid, goodsType string) (*model.CommCouponUserOrd, error) {
	var m model.CommCouponUserOrd
	has, err := session.Where("uid=? AND  type= ? and gid=? and goods_type=?", uid, types, gid, goodsType).Desc("batch").Get(&m)
	if has == false || err != nil {
		return nil, logx.Warn(err)
	}
	return &m, nil
}
func CouponUserOrdByOid(eg *xorm.Engine, uid int, oid string) float64 {
	var m model.CommCouponUserOrd
	total, err := eg.Where("uid=? AND type=? and  oid=? ", uid, 1, oid).Desc("batch").Sum(&m, "coupon_amount_value")
	if err != nil {
		return 0
	}
	return total
}
func CouponUserOrdByGidAscBatch(eg *xorm.Engine, uid int, gid string, refundState int, goodsType, pvd string) (*model.CommCouponUserOrd, error) {
	var m model.CommCouponUserOrd
	var session *xorm.Session
	if utils.InArr(pvd, []string{md.PVD_TB, md.PVD_TM}) {
		ex := strings.Split(gid, "-")
		if len(ex) == 2 {
			gid = ex[1]
			session = eg.Where("uid=? AND  type= ? and gid LIKE ? and refund_state=?", uid, 1, "%"+gid+"%", refundState)
		} else {
			session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=?", uid, 1, gid, refundState)
		}
	} else if utils.InArr(pvd, []string{md.PVD_JD}) && php2go.IsNumeric(gid) == false {
		ex := strings.Split(gid, "_")
		if len(ex) == 2 {
			gid = ex[1]
			session = eg.Where("uid=? AND  type= ? and gid LIKE ? and refund_state=?", uid, 1, "%"+gid+"%", refundState)
		} else {
			session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=?", uid, 1, gid, refundState)
		}
	} else {
		session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=?", uid, 1, gid, refundState)
	}
	if goodsType != "" {
		session = session.And("goods_type=?", goodsType)
	}
	has, err := session.Asc("batch").Get(&m)
	if has == false || err != nil {
		return nil, logx.Warn(err)
	}
	return &m, nil
}
func CouponUserOrdByGidAscBatchALL(eg *xorm.Engine, uid int, gid string, refundState int, batch int, goodsType, pvd string) (*[]model.CommCouponUserOrd, error) {
	var m []model.CommCouponUserOrd
	var session *xorm.Session
	if utils.InArr(pvd, []string{md.PVD_TB, md.PVD_TM}) {
		ex := strings.Split(gid, "-")
		if len(ex) == 2 {
			gid = ex[1]
			session = eg.Where("uid=? AND  type= ? and gid LIKE ? and refund_state=? and batch=?", uid, 1, "%"+gid+"%", refundState, batch)
		} else {
			session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=? and batch=?", uid, 1, gid, refundState, batch)
		}
	} else if utils.InArr(pvd, []string{md.PVD_JD}) && php2go.IsNumeric(gid) == false {
		ex := strings.Split(gid, "_")
		if len(ex) == 2 {
			gid = ex[1]
			session = eg.Where("uid=? AND  type= ? and gid LIKE ? and refund_state=? and batch=?", uid, 1, "%"+gid+"%", refundState, batch)
		} else {
			session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=? and batch=?", uid, 1, gid, refundState, batch)
		}
	} else {
		session = eg.Where("uid=? AND  type= ? and gid=? and refund_state=? and batch=?", uid, 1, gid, refundState, batch)
	}
	if goodsType != "" {
		session = session.And("goods_type=?", goodsType)
	}
	err := session.Find(&m)
	if err != nil {
		return nil, logx.Warn(err)
	}
	return &m, nil
}
func CouponUserOrdSum(Db *xorm.Engine, uid int, types, pvdType, times, cols string) (float64, error) {
	dbWhere := commWhere(Db, uid, types, pvdType, times)
	m := new(model.CommCouponUserOrd)
	total, err := dbWhere.Sum(m, cols)
	if err != nil {
		return 0, err
	}
	return total, err

}
