package taobao

import (
	"applet/app/utils"
	"applet/app/utils/cachesecond"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/syyongx/php2go"
	"time"
)

// 商品详情
func (t *TB) ItemDetailById(c *gin.Context, id, bizSceneId string) (*Material, error) {
	if php2go.IsNumeric(id) {
		return t.ItemDetailByLink(c, "https://item.taobao.com/item.htm?id="+id, "2")
	}
	return t.ItemDetailByLink(c, "https://uland.taobao.com/item/edetail?id="+id, "")
}
func (t *TB) ItemDetailByLink(c *gin.Context, l, bizSceneId string) (*Material, error) {
	var res *Material
	args := map[string]string{
		"q":         l,
		"adzone_id": AdZoneId(t.SelfPromoIdWeb), // 默认使用安卓, 转链时候才要针对不同的渠道
		"session":   t.SID,
		"page_size": "1",
		"page_no":   "1",
	}

	if bizSceneId == "2" {
		args["biz_scene_id"] = "2"
	}
	method := TBK_DG_MTL_OPT
	//判断是否跟随官方 需要换接口
	if t.AuthType == 1 {
		method = TBK_SC_MTL_OPT
		Sid := t.UserSID
		args["session"] = Sid
		_, args["site_id"], args["adzone_id"] = SplitPid(t.SelfPromoIdWeb)
	}
	args["relation_id"] = ""
	fmt.Println(c.GetString("taobao_share_id"))
	if c.GetString("taobao_share_id") != "" && c.GetString("taobao_share_id") != "0" {
		args["relation_id"] = c.GetString("taobao_share_id")
	}
	cacheKey := fmt.Sprintf("%s:tb_detail:%s_%s_%s_%s_%s", c.GetString("mid"), php2go.Md5(utils.SerializeStr(args)), method, t.Web_AK, t.Web_SK, args["session"])
	err := cachesecond.GetJson(cacheKey, &res)
	if err != nil || res == nil {
		resp, err := send(args, method, t.Web_AK, t.Web_SK)
		if err != nil {
			return nil, err
		}
		fmt.Println("淘宝", args)
		fmt.Println("淘宝", string(resp))
		// todo 可能还需返回的说明, 里面包含详细介绍的图片
		// "https://mdetail.tmall.com/templates/pages/desc?id=" + id
		// "https://h5.m.taobao.com/app/detail/desc.html?type=1&f=&sellerType=B&_isH5Des=true#!id=" + id
		// 如果是跟随官方 结构不一样

		if t.AuthType == 1 {
			var tmp scTbkScMaterialOptionalUpgradeResponse
			if err = json.Unmarshal(resp, &tmp); err != nil {
				return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
			}
			if len(tmp.TbkScMaterialOptionalUpgradeResponse.ResultList.MapData) == 0 {
				return nil, logx.Warn("<resp no data>: " + string(resp))
			}
			data := tmp.TbkScMaterialOptionalUpgradeResponse.ResultList.MapData[0]
			tmp1 := Material{
				CategoryID:     data.ItemBasicInfo.CategoryId,
				CategoryName:   data.ItemBasicInfo.CategoryName,
				CommissionRate: utils.Float64ToStr(utils.StrToFloat64(data.PublishInfo.IncomeRate) * 100),
				ItemID:         data.ItemId,
				Nick:           data.ItemBasicInfo.ShopTitle,
				PictURL:        data.ItemBasicInfo.PictUrl,
				SellerID:       int(data.ItemBasicInfo.SellerId),
				ShopTitle:      data.ItemBasicInfo.ShopTitle,
				ShortTitle:     data.ItemBasicInfo.ShortTitle,
				Title:          data.ItemBasicInfo.Title,
				ZkFinalPrice:   data.PricePromotionInfo.ZkFinalPrice,
				ReservePrice:   data.PricePromotionInfo.ReservePrice,
				Volume:         data.ItemBasicInfo.Volume,
				UserType:       data.ItemBasicInfo.UserType,
				TkTotalSales:   utils.IntToStr(data.ItemBasicInfo.Volume),
			}
			tmp1.SmallImages = data.ItemBasicInfo.SmallImages
			if len(data.ItemBasicInfo.SmallImages.String) == 0 {
				tmp1.SmallImages.String = []string{tmp1.PictURL}
			}
			if utils.StrToFloat64(data.PricePromotionInfo.FinalPromotionPrice) > 0 {

				for _, v1 := range data.PricePromotionInfo.FinalPromotionPathList.FinalPromotionPathMapData {
					tmp1.CouponAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(v1.PromotionFee)+utils.StrToFloat64(tmp1.CouponAmount), 2)
					tmp1.CouponStartTime = utils.IntToStr(utils.StrToInt(v1.PromotionStartTime) / 1000)
					tmp1.CouponEndTime = utils.IntToStr(utils.StrToInt(v1.PromotionEndTime) / 1000)
					tmp1.CouponID = v1.PromotionId
					if v1.PromotionTitle == "商品券" || v1.PromotionTitle == "店铺券" {
						if utils.StrToInt64(v1.PromotionEndTime)/1000 > time.Now().Unix() {
							tmp1.GoodsCoupon = "1"
						}
					}
				}
			}
			cachesecond.SetJson(cacheKey, tmp1, 30)
			return &tmp1, nil
		} else {
			var tmp wrapperStruct
			if err = json.Unmarshal(resp, &tmp); err != nil {
				return nil, err

			}
			if len(tmp.TbkDgMaterialOptionalResponse.ResultList.MapData) == 0 {
				return nil, err
			}
			data := tmp.TbkDgMaterialOptionalResponse.ResultList.MapData[0]
			itemID, ok := data.ItemID.(float64)
			if ok {
				data.ItemID = int64(itemID)
			}
			cachesecond.SetJson(cacheKey, data, 30)
			return &data, nil
		}
	}
	return res, nil
}

func (t *TB) ItemDetailByIdKey(c *gin.Context, id, bizSceneId string) (*Material, error) {
	var res *Material
	args := map[string]string{
		"item_id":  id,
		"session":  t.SID,
		"platform": "2",
	}
	if php2go.IsNumeric(id) {
		args["biz_scene_id"] = "2"
	}
	if bizSceneId != "" {
		args["biz_scene_id"] = bizSceneId
	}
	method := "taobao.tbk.item.info.upgrade.get"
	//判断是否跟随官方 需要换接口
	if t.AuthType == 1 {
		Sid := t.UserSID
		args["session"] = Sid
	}
	cacheKey := fmt.Sprintf("%s:tb_detail_key:%s_%s_%s_%s_%s", c.GetString("mid"), php2go.Md5(utils.SerializeStr(args)), method, t.Web_AK, t.Web_SK, args["session"])
	err := cachesecond.GetJson(cacheKey, &res)
	if err != nil || res == nil {
		resp, err := send(args, method, t.Web_AK, t.Web_SK)
		if err != nil {
			return nil, err
		}
		var tmp scTbkItemInfoUpgradeGetResponse
		if err = json.Unmarshal(resp, &tmp); err != nil {
			return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
		}
		if len(tmp.TbkItemInfoUpgradeGetResponse.Results.TbkItemDetail) == 0 {
			return nil, err
		}
		data := tmp.TbkItemInfoUpgradeGetResponse.Results.TbkItemDetail[0]
		tmp1 := Material{
			CategoryID:     data.ItemBasicInfo.CategoryId,
			CategoryName:   data.ItemBasicInfo.CategoryName,
			CommissionRate: utils.Float64ToStr(utils.StrToFloat64(data.PublishInfo.IncomeRate) * 100),
			ItemID:         data.ItemId,
			Nick:           data.ItemBasicInfo.ShopTitle,
			PictURL:        data.ItemBasicInfo.PictUrl,
			SellerID:       int(data.ItemBasicInfo.SellerId),
			ShopTitle:      data.ItemBasicInfo.ShopTitle,
			ShortTitle:     data.ItemBasicInfo.ShortTitle,
			Title:          data.ItemBasicInfo.Title,
			ZkFinalPrice:   data.PricePromotionInfo.ZkFinalPrice,
			ReservePrice:   data.PricePromotionInfo.ReservePrice,
			Volume:         data.ItemBasicInfo.Volume,
		}
		tmp1.SmallImages = data.ItemBasicInfo.SmallImages
		if len(data.ItemBasicInfo.SmallImages.String) == 0 {
			tmp1.SmallImages.String = []string{tmp1.PictURL}
		}
		if utils.StrToFloat64(data.PricePromotionInfo.FinalPromotionPrice) > 0 {

			for _, v1 := range data.PricePromotionInfo.FinalPromotionPathList.FinalPromotionPathMapData {
				tmp1.CouponAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(v1.PromotionFee)+utils.StrToFloat64(tmp1.CouponAmount), 2)
				tmp1.CouponStartTime = utils.IntToStr(utils.StrToInt(v1.PromotionStartTime) / 1000)
				tmp1.CouponEndTime = utils.IntToStr(utils.StrToInt(v1.PromotionEndTime) / 1000)
				tmp1.CouponID = v1.PromotionId
				if v1.PromotionTitle == "商品券" || v1.PromotionTitle == "店铺券" {
					if utils.StrToInt64(v1.PromotionEndTime)/1000 > time.Now().Unix() {
						tmp1.GoodsCoupon = "1"
					}
				}
			}
		}
		cachesecond.SetJson(cacheKey, tmp1, 60*3)
		return &tmp1, nil
	}
	return res, nil
}
