package taobao

import (
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
)

type PublisherInfoListData struct {
	TbkScPublisherInfoGetResponse struct {
		Data struct {
			TotalCount  int `json:"total_count"`
			InviterList struct {
				MapData []struct {
					RelationApp  string `json:"relation_app"`
					CreateDate   string `json:"create_date"`
					AccountName  string `json:"account_name"`
					RealName     string `json:"real_name"`
					RelationId   int    `json:"relation_id"`
					OfflineScene string `json:"offline_scene"`
					OnlineScene  string `json:"online_scene"`
					Note         string `json:"note"`
					RootPid      string `json:"root_pid"`
					Rtag         string `json:"rtag"`
					OfflineInfo  struct {
						ShopName        string `json:"shop_name"`
						ShopType        string `json:"shop_type"`
						PhoneNumber     string `json:"phone_number"`
						DetailAddress   string `json:"detail_address"`
						Location        string `json:"location"`
						ShopCertifyType string `json:"shop_certify_type"`
						CertifyNumber   string `json:"certify_number"`
						Career          string `json:"career"`
					} `json:"offline_info"`
					SpecialId    int    `json:"special_id"`
					PunishStatus string `json:"punish_status"`
					ExternalId   string `json:"external_id"`
					ExternalType string `json:"external_type"`
				} `json:"map_data"`
			} `json:"inviter_list"`
			RootPidChannelList struct {
				String []string `json:"string"`
			} `json:"root_pid_channel_list"`
		} `json:"data"`
	} `json:"tbk_sc_publisher_info_get_response"`
}

func (t *TB) PublisherInfoList(p string) (*[]md.PublisherInfo, error) {
	method := "taobao.tbk.sc.publisher.info.get"
	args := map[string]string{
		"relation_app": "common",
		"session":      t.SID, //
		"info_type":    "2",
		"page_no":      p,
		"page_size":    "10",
	}
	resp, err := send(args, method, t.Svc_AK, t.Svc_SK)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp PublisherInfoListData
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, err
	}
	data := tmp.TbkScPublisherInfoGetResponse.Data.InviterList.MapData
	var list = make([]md.PublisherInfo, 0)
	for _, v := range data {
		if v.ExternalId == "" {
			v.ExternalId = v.Rtag
		}
		if v.ExternalId == "" {
			continue
		}
		var tmpData = md.PublisherInfo{
			RelationId: utils.IntToStr(v.RelationId),
			SpecialId:  utils.IntToStr(v.SpecialId),
			ExternalId: v.ExternalId,
		}
		list = append(list, tmpData)
	}
	return &list, nil
}
func (t *TB) PublisherInfoRelationList(types, p string) (*[]md.PublisherInfo, error) {
	method := "taobao.tbk.sc.publisher.info.get"
	args := map[string]string{
		"relation_app": "common",
		"session":      t.SID, //
		"info_type":    "1",
		"page_no":      p,
		"page_size":    "100",
	}
	resp, err := send(args, method, t.Svc_AK, t.Svc_SK)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp PublisherInfoListData
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, err
	}
	data := tmp.TbkScPublisherInfoGetResponse.Data.InviterList.MapData
	var list = make([]md.PublisherInfo, 0)
	for _, v := range data {
		if v.ExternalId == "" {
			v.ExternalId = v.Rtag
		}
		if v.Rtag == "" && types == "2" {
			v.Rtag = v.Note
		}
		if v.ExternalId == "" && types == "1" {
			continue
		}
		if v.RelationId == 0 && types == "2" {
			continue
		}
		var tmpData = md.PublisherInfo{
			RelationId: utils.IntToStr(v.RelationId),
			SpecialId:  utils.IntToStr(v.SpecialId),
			ExternalId: v.ExternalId,
		}
		if types == "2" {
			tmpData.Rtag = v.Rtag
		}
		list = append(list, tmpData)
	}
	return &list, nil
}
