package taobao

import (
	"encoding/json"
	"fmt"
	"time"

	"applet/app/utils"
	"applet/app/utils/logx"
)

type RefundOrder struct {
	AlscID                      string `json:"alsc_id"`
	AlscPid                     string `json:"alsc_pid"`
	EarningTime                 string `json:"earning_time"`
	ModifiedTime                string `json:"modified_time"`
	RefundFee                   string `json:"refund_fee"`
	RefundStatus                int64  `json:"refund_status"`
	RefundType                  int64  `json:"refund_type"`
	RelationID                  int64  `json:"relation_id"`
	SpecialID                   int64  `json:"special_id"`
	TbAuctionTitle              string `json:"tb_auction_title"`
	TbTradeCreateTime           string `json:"tb_trade_create_time"`
	TbTradeFinishPrice          string `json:"tb_trade_finish_price"`
	TbTradeID                   int64  `json:"tb_trade_id"`
	TbTradeParentID             int64  `json:"tb_trade_parent_id"`
	Tk3rdPubID                  int64  `json:"tk3rd_pub_id"`
	Tk3rdPubShowReturnFee       string `json:"tk3rd_pub_show_return_fee"`
	TkCommissionFeeRefund3rdPub string `json:"tk_commission_fee_refund3rd_pub"`
	TkCommissionFeeRefundPub    string `json:"tk_commission_fee_refund_pub"`
	TkPubID                     int64  `json:"tk_pub_id"`
	TkPubShowReturnFee          string `json:"tk_pub_show_return_fee"`
	TkRefundSuitTime            string `json:"tk_refund_suit_time"`
	TkRefundTime                string `json:"tk_refund_time"`
	TkSubsidyFeeRefund3rdPub    string `json:"tk_subsidy_fee_refund3rd_pub"`
	TkSubsidyFeeRefundPub       string `json:"tk_subsidy_fee_refund_pub"`
}

// 淘宝维权订单
func (t *TB) RefundOrder(pageNo, pageSize int, sTime, eTime int64, args map[string]string) (*[]RefundOrder, error) {

	// 默认查询20分钟内数据
	//sTime = utils.TimeStdParseUnix("2021-07-16 20:11:26")

	st, _ := time.Unix(sTime, 0).Format("2006-01-02 15:04:05"), time.Unix(eTime, 0).Format("2006-01-02 15:04:05")
	//fmt.Printf("******************sid: %v, ak: %s, sk: %v*******************", t.Svc_SID, t.Svc_AK, t.Svc_SK)
	method := "taobao.tbk.sc.relation.refund"
	searchOption := map[string]string{
		"page_no":     utils.IntToStr(pageNo),
		"page_size":   utils.IntToStr(pageSize),
		"search_type": "5",              //1-维权发起时间，2-订单结算时间（正向订单），3-维权完成时间，4-订单创建时间，5-订单更新时间
		"refund_type": "0",              //1 表示2方，2表示3方，0表示不限
		"start_time":  st,               // 2019-04-05 12:18:22
		"biz_type":    args["biz_type"], //1代表渠道关系id，2代表会员关系id
	}
	params := map[string]string{
		"session":       t.SID,
		"search_option": string(utils.Serialize(searchOption)),
	}
	resp, err := send(params, method, t.Svc_AK, t.Svc_SK)
	fmt.Println(params)
	fmt.Println(string(resp))
	if err != nil {
		return nil, logx.Warn(err)
	}

	tmp := struct {
		TbkScRelationRefundResponse struct {
			Result struct {
				BizErrorCode int64  `json:"biz_error_code"`
				BizErrorDesc string `json:"biz_error_desc"`
				Data         struct {
					PageNo   int64 `json:"page_no"`
					PageSize int64 `json:"page_size"`
					Results  struct {
						Result []RefundOrder `json:"result"`
					} `json:"results"`
					TotalCount int64 `json:"total_count"`
				} `json:"data"`
				ResultCode int64  `json:"result_code"`
				ResultMsg  string `json:"result_msg"`
			} `json:"result"`
		} `json:"tbk_sc_relation_refund_response"`
	}{}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn(resp)
	}
	if len(tmp.TbkScRelationRefundResponse.Result.Data.Results.Result) == 0 {
		return &[]RefundOrder{}, nil
	}
	var orderInfo = tmp.TbkScRelationRefundResponse.Result.Data.Results.Result
	return &orderInfo, err
}
