package taobao

import (
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"time"
)

// 淘宝物料
type Material struct {
	CategoryID           int         `json:"category_id"`
	CategoryName         string      `json:"category_name"`
	CommissionRate       string      `json:"commission_rate"`
	CommissionType       string      `json:"commission_type"`
	CouponID             string      `json:"coupon_id"`
	GoodsCoupon          string      `json:"goods_coupon"`
	CouponInfo           string      `json:"coupon_info"`
	CouponRemainCount    int         `json:"coupon_remain_count"`
	CouponTotalCount     int         `json:"coupon_total_count"`
	IncludeDxjh          string      `json:"include_dxjh"`
	IncludeMkt           string      `json:"include_mkt"`
	InfoDxjh             string      `json:"info_dxjh"`
	ItemDescription      string      `json:"item_description"`
	ItemID               interface{} `json:"item_id"`
	ItemURL              string      `json:"item_url"`
	LevelOneCategoryID   int         `json:"level_one_category_id"`
	LevelOneCategoryName string      `json:"level_one_category_name"`
	Nick                 string      `json:"nick"`
	PictURL              string      `json:"pict_url"`
	Provcity             string      `json:"provcity"`
	RealPostFee          string      `json:"real_post_fee"`
	ReservePrice         string      `json:"reserve_price"`
	SellerID             int         `json:"seller_id"`
	ShopDsr              int         `json:"shop_dsr"`
	ShopTitle            string      `json:"shop_title"`
	ShortTitle           string      `json:"short_title"`
	ActivityId           string      `json:"activity_id"`
	SmallImages          struct {
		String []string `json:"string"`
	} `json:"small_images"`
	Title           string      `json:"title"`
	TkTotalCommi    string      `json:"tk_total_commi"`
	TkTotalSales    string      `json:"tk_total_sales"`
	URL             string      `json:"url"`
	UserType        int         `json:"user_type"`
	Volume          int         `json:"volume"`
	WhiteImage      string      `json:"white_image"`
	XID             string      `json:"x_id"`
	ZkFinalPrice    string      `json:"zk_final_price"`
	CouponAmount    string      `json:"coupon_amount,omitempty"`
	OldCouponAmount string      `json:"old_coupon_amount"`
	CouponEndTime   string      `json:"coupon_end_time,omitempty"`
	CouponShareURL  string      `json:"coupon_share_url,omitempty"`
	CouponStartFee  string      `json:"coupon_start_fee,omitempty"`
	CouponStartTime string      `json:"coupon_start_time,omitempty"`
	InputNumIid     interface{} `json:"input_num_iid"`
}

type wrapperStruct struct {
	TbkDgMaterialOptionalResponse struct {
		ResultList struct {
			MapData []Material `json:"map_data"`
		} `json:"result_list"`
	} `json:"tbk_dg_material_optional_response"`
}
type tbDetailStruct struct {
	TbkItemInfoGetResponse struct {
		Results struct {
			NBbkItem []Material `json:"n_tbk_item"`
		} `json:"results"`
	} `json:"tbk_item_info_get_response"`
}
type scTbkItemInfoUpgradeGetResponse struct {
	TbkItemInfoUpgradeGetResponse struct {
		Results struct {
			TbkItemDetail []NewMarterial `json:"tbk_item_detail"`
		} `json:"results"`
	} `json:"tbk_item_info_upgrade_get_response"`
}
type scMaterialOptionalStruct struct {
	TbkScMaterialOptionalResponse struct {
		ResultList struct {
			MapData []Material `json:"map_data"`
		} `json:"result_list"`
	} `json:"tbk_sc_material_optional_response"`
}

type NewMarterial struct {
	ItemBasicInfo struct {
		BrandName            string `json:"brand_name"`
		CategoryId           int    `json:"category_id"`
		CategoryName         string `json:"category_name"`
		LevelOneCategoryId   int    `json:"level_one_category_id"`
		LevelOneCategoryName string `json:"level_one_category_name"`
		PictUrl              string `json:"pict_url"`
		SellerId             int64  `json:"seller_id"`
		ShopTitle            string `json:"shop_title"`
		ShortTitle           string `json:"short_title"`
		SubTitle             string `json:"sub_title"`
		Title                string `json:"title"`
		UserType             int    `json:"user_type"`
		Volume               int    `json:"volume"`
		TkTotalSales         string `json:"tk_total_sales"`
		WhiteImage           string `json:"white_image"`
		SmallImages          struct {
			String []string `json:"string"`
		} `json:"small_images"`
	} `json:"item_basic_info"`
	ItemId      string `json:"item_id"`
	PresaleInfo struct {
		PresaleDeposit string `json:"presale_deposit"`
	} `json:"presale_info"`
	PricePromotionInfo struct {
		FinalPromotionPathList struct {
			FinalPromotionPathMapData []struct {
				PromotionDesc      string `json:"promotion_desc"`
				PromotionEndTime   string `json:"promotion_end_time"`
				PromotionFee       string `json:"promotion_fee"`
				PromotionId        string `json:"promotion_id"`
				PromotionStartTime string `json:"promotion_start_time"`
				PromotionTitle     string `json:"promotion_title"`
			} `json:"final_promotion_path_map_data"`
		} `json:"final_promotion_path_list"`
		FinalPromotionPrice string `json:"final_promotion_price"`
		MorePromotionList   struct {
			MorePromotionMapData []struct {
				PromotionDesc      string `json:"promotion_desc"`
				PromotionEndTime   string `json:"promotion_end_time"`
				PromotionStartTime string `json:"promotion_start_time"`
				PromotionTitle     string `json:"promotion_title"`
			} `json:"more_promotion_map_data"`
		} `json:"more_promotion_list,omitempty"`
		PredictRoundingUpPathList struct {
			PredictRoundingUpPathMapData []struct {
				PromotionDesc  string `json:"promotion_desc"`
				PromotionTitle string `json:"promotion_title"`
			} `json:"predict_rounding_up_path_map_data"`
		} `json:"predict_rounding_up_path_list,omitempty"`
		PredictRoundingUpPrice     string `json:"predict_rounding_up_price,omitempty"`
		PredictRoundingUpPriceDesc string `json:"predict_rounding_up_price_desc,omitempty"`
		PromotionTagList           struct {
			PromotionTagMapData []struct {
				TagName string `json:"tag_name"`
			} `json:"promotion_tag_map_data"`
		} `json:"promotion_tag_list"`
		ReservePrice string `json:"reserve_price"`
		ZkFinalPrice string `json:"zk_final_price"`
	} `json:"price_promotion_info"`
	PublishInfo struct {
		ClickUrl       string `json:"click_url"`
		CouponShareUrl string `json:"coupon_share_url"`
		IncomeInfo     struct {
			CommissionAmount string `json:"commission_amount"`
			CommissionRate   string `json:"commission_rate"`
			SubsidyAmount    string `json:"subsidy_amount"`
			SubsidyRate      string `json:"subsidy_rate"`
		} `json:"income_info"`
		IncomeRate string `json:"income_rate"`
	} `json:"publish_info"`
	ScopeInfo struct {
		SuperiorBrand interface{} `json:"superior_brand"`
	} `json:"scope_info"`
}
type scTbkScMaterialOptionalUpgradeResponse struct {
	TbkScMaterialOptionalUpgradeResponse struct {
		TotalResults int `json:"total_results"`
		ResultList   struct {
			MapData []NewMarterial `json:"map_data"`
		} `json:"result_list"`
		UvidMsg string `json:"uvid_msg"`
	} `json:"tbk_sc_material_optional_upgrade_response"`
}
type ownTbkDgMaterialOptionalUpgradeResponse struct {
	TbkDgMaterialOptionalUpgradeResponse struct {
		TotalResults int `json:"total_results"`
		ResultList   struct {
			MapData []NewMarterial `json:"map_data"`
		} `json:"result_list"`
		UvidMsg string `json:"uvid_msg"`
	} `json:"tbk_dg_material_optional_upgrade_response"`
}
type featuredWrapperStruct struct {
	TbkDgMaterialOptionalResponse struct {
		ResultList struct {
			MapData []TBSearchFeatureStruct `json:"map_data"`
		} `json:"result_list"`
	} `json:"tbk_dg_material_optional_response"`
}
type featuredScMaterialOptionalStruct struct {
	TbkScMaterialOptionalResponse struct {
		ResultList struct {
			MapData []TBSearchFeatureStruct `json:"map_data"`
		} `json:"result_list"`
	} `json:"tbk_sc_material_optional_response"`
}

var sortList = map[string]struct{}{
	"total_sales_asc":    {}, //销量从低到高
	"total_sales_des":    {}, //销量从高到低
	"price_asc":          {}, //价格从低到高
	"price_des":          {}, //价格从高到低
	"tk_rate_des":        {}, //佣金比率从高到低
	"tk_rate_asc":        {}, //佣金比率从低到高
	"tk_total_sales_asc": {}, //累计推广量从低到高
	"tk_total_sales_des": {}, //累计推广量成高到低
	"tk_total_commi_asc": {}, //总支出佣金从低到高
	"tk_total_commi_des": {}, //总支出佣金高到低
}

func (t *TB) GetList(c *gin.Context, tbArgs TaoBaoArgs, bizSceneId string) ([]byte, error) {
	args := map[string]string{
		"q":         tbArgs.Keyword,
		"adzone_id": AdZoneId(t.SelfPromoIdWeb),
		"page_size": utils.IntToStr(utils.StrToInt(tbArgs.PageSize)),
		"page_no":   utils.IntToStr(utils.StrToInt(tbArgs.PageIndex)),
		"platform":  "2",         // 链接形式-1：PC,2：无线,默认为１
		"sort":      tbArgs.Sort, // 排序_des(降序),排序_asc(升序),销量(total_sales)淘客佣金比率(tk_rate)累计推广量(tk_total_sales),总支出佣金(tk_total_commi),价格(price)

		//"site_id" : "",
	}
	if bizSceneId != "" {
		args["biz_scene_id"] = "2"
	}
	method := TBK_DG_MTL_OPT
	//判断是否跟随官方 需要换接口
	if t.AuthType == 1 {
		method = TBK_SC_MTL_OPT
		Sid := t.UserSID
		args["session"] = Sid
		_, args["site_id"], args["adzone_id"] = SplitPid(t.SelfPromoIdWeb)
	}
	// TODO:  args[sort] 修复为 args["sort"]
	if _, ok := args["sort"]; !ok {
		// TODO 按销量高到低排序 而不是 推广量高到低排序
		// args["sort"] = "tk_total_sales_des"
		// TODO 现修改为：按销量高到低排序
		args["sort"] = "total_sales_des"
	}
	if args["sort"] == "" {
		args["sort"] = "total_sales_des"
	}
	if utils.StrToFloat64(tbArgs.StartPrice) > 0 {
		// TODO 数据库中存储的类型为decimal(11,2)（decimal表现形式类似浮点型，但底层结构不一样），无法正常将数据库中的decimal字符串转换int
		// args["start_price"] = utils.IntToStr(utils.StrToInt(startPrice))
		// TODO 修复为：
		args["start_price"] = fmt.Sprintf("%.0f", utils.StrToFloat64(tbArgs.StartPrice)) //utils.IntToStr(utils.StrToInt(startPrice))
	}
	if utils.StrToFloat64(tbArgs.StartCommissionRate) > 0 {
		args["start_tk_rate"] = tbArgs.StartCommissionRate
	}
	if utils.StrToFloat64(tbArgs.EndCommissionRate) > 0 {
		args["end_tk_rate"] = tbArgs.EndCommissionRate
	}
	if utils.StrToFloat64(tbArgs.EndPrice) > 0 {
		args["end_price"] = utils.IntToStr(utils.StrToInt(tbArgs.EndPrice))
	}
	if tbArgs.IsTmall == "true" {
		args["is_tmall"] = "true"
	}
	if tbArgs.IsCoupon == "true" {
		args["has_coupon"] = tbArgs.IsCoupon
	}
	args["relation_id"] = ""
	if c.GetString("taobao_share_id") != "" && c.GetString("taobao_share_id") != "0" {
		args["relation_id"] = c.GetString("taobao_share_id")
	}
	resp, err := send(args, method, t.Web_AK, t.Web_SK)
	if err != nil {
		return []byte(""), err
	}
	return resp, nil
}
func (t *TB) SearchList(c *gin.Context, args TaoBaoArgs) (*[]Material, error) {
	resp, err := t.GetList(c, args, args.Biz)
	//如果是跟随官方 结构不一样
	if t.AuthType == 1 {
		var tmp scTbkScMaterialOptionalUpgradeResponse
		if err = json.Unmarshal(resp, &tmp); err != nil {
			return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
		}
		if len(tmp.TbkScMaterialOptionalUpgradeResponse.ResultList.MapData) == 0 {
			return nil, logx.Warn("<resp no data>: " + string(resp))
		}
		data := tmp.TbkScMaterialOptionalUpgradeResponse.ResultList.MapData
		list := make([]Material, 0)
		for _, v := range data {
			tmp1 := Material{
				CategoryID:     v.ItemBasicInfo.CategoryId,
				CategoryName:   v.ItemBasicInfo.CategoryName,
				CommissionRate: utils.Float64ToStr(utils.StrToFloat64(v.PublishInfo.IncomeRate) * 100),
				ItemID:         v.ItemId,
				Nick:           v.ItemBasicInfo.ShopTitle,
				PictURL:        v.ItemBasicInfo.PictUrl,
				SellerID:       int(v.ItemBasicInfo.SellerId),
				ShopTitle:      v.ItemBasicInfo.ShopTitle,
				ShortTitle:     v.ItemBasicInfo.ShortTitle,
				Title:          v.ItemBasicInfo.Title,
				ZkFinalPrice:   v.PricePromotionInfo.ZkFinalPrice,
				ReservePrice:   v.PricePromotionInfo.ReservePrice,
				Volume:         v.ItemBasicInfo.Volume,
			}
			tmp1.SmallImages.String = []string{tmp1.PictURL}

			if utils.StrToFloat64(v.PricePromotionInfo.FinalPromotionPrice) > 0 {
				for _, v1 := range v.PricePromotionInfo.FinalPromotionPathList.FinalPromotionPathMapData {
					tmp1.CouponAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(v1.PromotionFee)+utils.StrToFloat64(tmp1.CouponAmount), 2)
					tmp1.CouponStartTime = utils.IntToStr(utils.StrToInt(v1.PromotionStartTime) / 1000)
					tmp1.CouponEndTime = utils.IntToStr(utils.StrToInt(v1.PromotionEndTime) / 1000)
					tmp1.CouponID = v1.PromotionId
					if v1.PromotionTitle == "商品券" || v1.PromotionTitle == "店铺券" {
						if utils.StrToInt64(v1.PromotionEndTime)/1000 > time.Now().Unix() {
							tmp1.GoodsCoupon = "1"
						}
					}
				}
			}
			list = append(list, tmp1)
		}
		return &list, nil
	} else {
		var tmp ownTbkDgMaterialOptionalUpgradeResponse
		if err = json.Unmarshal(resp, &tmp); err != nil {
			return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
		}
		if len(tmp.TbkDgMaterialOptionalUpgradeResponse.ResultList.MapData) == 0 {
			return nil, logx.Warn("<resp no data>: " + string(resp))
		}
		data := tmp.TbkDgMaterialOptionalUpgradeResponse.ResultList.MapData
		list := make([]Material, 0)
		for _, v := range data {
			tmp1 := Material{
				CategoryID:     v.ItemBasicInfo.CategoryId,
				CategoryName:   v.ItemBasicInfo.CategoryName,
				CommissionRate: utils.Float64ToStr(utils.StrToFloat64(v.PublishInfo.IncomeRate) * 100),
				ItemID:         v.ItemId,
				Nick:           v.ItemBasicInfo.ShopTitle,
				PictURL:        v.ItemBasicInfo.PictUrl,
				SellerID:       int(v.ItemBasicInfo.SellerId),
				ShopTitle:      v.ItemBasicInfo.ShopTitle,
				ShortTitle:     v.ItemBasicInfo.ShortTitle,
				Title:          v.ItemBasicInfo.Title,
				ZkFinalPrice:   v.PricePromotionInfo.ZkFinalPrice,
				ReservePrice:   v.PricePromotionInfo.ReservePrice,
				Volume:         v.ItemBasicInfo.Volume,
			}
			tmp1.SmallImages.String = []string{tmp1.PictURL}
			if utils.StrToFloat64(v.PricePromotionInfo.FinalPromotionPrice) > 0 {

				for _, v1 := range v.PricePromotionInfo.FinalPromotionPathList.FinalPromotionPathMapData {
					tmp1.CouponAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(v1.PromotionFee)+utils.StrToFloat64(tmp1.CouponAmount), 2)

					tmp1.CouponStartTime = utils.IntToStr(utils.StrToInt(v1.PromotionStartTime) / 1000)
					tmp1.CouponEndTime = utils.IntToStr(utils.StrToInt(v1.PromotionEndTime) / 1000)
					tmp1.CouponID = v1.PromotionId
					if v1.PromotionTitle == "商品券" || v1.PromotionTitle == "店铺券" {
						if utils.StrToInt64(v1.PromotionEndTime)/1000 > time.Now().Unix() {
							tmp1.GoodsCoupon = "1"
						}
					}

				}
			}
			list = append(list, tmp1)
		}
		return &list, nil
	}

}

func (t *TB) FeaturedList(c *gin.Context, args TaoBaoArgs) (*[]TBFeatureStruct, error) {
	resp, err := t.GetList(c, args, "")
	//如果是跟随官方 结构不一样
	if t.AuthType == 1 {
		var tmp featuredScMaterialOptionalStruct
		if err = json.Unmarshal(resp, &tmp); err != nil {
			return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
		}
		if len(tmp.TbkScMaterialOptionalResponse.ResultList.MapData) == 0 {
			return nil, logx.Warn("<resp no data>: " + string(resp))
		}
		var data = tmp.TbkScMaterialOptionalResponse.ResultList.MapData
		var res []TBFeatureStruct
		for _, v := range data {
			var tmp TBFeatureStruct
			copier.Copy(&tmp, v)
			tmp.ItemID = utils.AnyToString(v.ItemID)
			tmp.CouponAmount = utils.StrToFloat64(v.CouponAmount)
			res = append(res, tmp)
		}
		return &res, nil
	} else {
		var tmp featuredWrapperStruct
		if err = json.Unmarshal(resp, &tmp); err != nil {
			return nil, logx.Warn("<resp>: " + string(resp) + ", err: " + err.Error())
		}
		if len(tmp.TbkDgMaterialOptionalResponse.ResultList.MapData) == 0 {
			return nil, logx.Warn("<resp no data>: " + string(resp))
		}
		var data = tmp.TbkDgMaterialOptionalResponse.ResultList.MapData
		var res []TBFeatureStruct
		for _, v := range data {
			var tmp TBFeatureStruct
			copier.Copy(&tmp, v)
			tmp.ItemID = utils.AnyToString(v.ItemID)
			tmp.CouponAmount = utils.StrToFloat64(v.CouponAmount)
			res = append(res, tmp)
		}
		return &res, nil
	}

}
