package taobao

import (
	"applet/app/e"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"github.com/tidwall/gjson"
	"time"
)

type TljInfo struct {
	AvailableFee string `json:"available_fee"`
	RightsID     string `json:"rights_id"`
	SendURL      string `json:"send_url"`
	VegasCode    string `json:"vegas_code"`
}

// 淘礼金创建
func (t *TB) TljCreate(itemId, money string, pid string, endTime string) (*TljInfo, error) {
	//https://open.taobao.com/api.htm?spm=a219a.7386797.0.0.b68a669aMDZlG7&source=search&docId=40173&docType=2#responseParams
	method := "taobao.tbk.dg.vegas.tlj.create"
	_, siteId, adZoneId := SplitPid(pid)
	args := map[string]string{
		"item_id":                  itemId,
		"session":                  t.UserSID, // 一般用站长就可以转链
		"adzone_id":                adZoneId,
		"site_id":                  siteId,
		"name":                     "淘礼金来啦",
		"total_num":                "1",    //淘礼金总个数
		"user_total_win_num_limit": "1",    //单用户累计中奖次数上限
		"per_face":                 money,  //单个淘礼金面额，支持两位小数，单位元
		"security_switch":          "true", //启动安全：true；不启用安全：false
		"send_start_time":          utils.TimeToStr(time.Now().Unix()),
		"send_end_time":            endTime,
		"use_end_time":             "7", //	使用结束日期。如果是结束时间模式为相对时间，时间格式为1-7直接的整数, 例如，1（相对领取时间1天）； 如果是绝对时间，格式为yyyy-MM-dd，例如，2019-01-29，表示到2019-01-29 23:59:59结束
		"use_end_time_mode":        "1", //	结束日期的模式,1:相对时间，2:绝对时间
		"security_level":           "0", //安全等级，0：适用于常规淘礼金投放场景；1：更高安全级别，适用于淘礼金面额偏大等安全性较高的淘礼金投放场景，可能导致更多用户拦截。security_switch为true，此字段不填写时，使用0作为默认安全级别。如果security_switch为false，不进行安全判断。
	}

	resp, err := send(args, method, t.Web_AK, t.Web_SK)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		TbkDgVegasTljCreateResponse struct {
			Result struct {
				Model   TljInfo `json:"model"`
				MsgCode string  `json:"msg_code"`
				MsgInfo string  `json:"msg_info"`
				Success bool    `json:"success"`
			} `json:"result"`
		} `json:"tbk_dg_vegas_tlj_create_response"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, err
	}
	if tmp.TbkDgVegasTljCreateResponse.Result.Model.SendURL == "" {
		logx.Warn(resp)
		return nil, e.NewErr(400, gjson.GetBytes(resp, "tbk_dg_vegas_tlj_create_response.result.msg_info").String())
	}
	return &tmp.TbkDgVegasTljCreateResponse.Result.Model, nil
}
