package taobao

import (
	"encoding/json"
	"regexp"

	"applet/app/utils/logx"
)

type WordItem struct {
	Content     string `json:"content"`
	NativeURL   string `json:"native_url"`
	PicURL      string `json:"pic_url"`
	Price       string `json:"price"`
	ThumbPicURL string `json:"thumb_pic_url"`
	Title       string `json:"title"`
	URL         string `json:"url"`
	ItemID      string `json:"item_id"`
}

// TODO 淘口令解析, 因为目前没有可用账号, 暂时搁置, 请查看智盟的接口
func (t *TB) WordParse(s string) (*WordItem, error) {
	method := "taobao.wireless.share.tpwd.query"
	args := map[string]string{"password_content": s}
	resp, err := send(args, method, t.Web_AK, t.Web_SK)
	if err != nil {
		return nil, logx.Warn("[resp]:" + string(resp) + ", [err]:" + err.Error())
	}
	var tmp struct {
		WirelessShareTpwdQueryResponse WordItem `json:"wireless_share_tpwd_query_response"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil || tmp.WirelessShareTpwdQueryResponse.Content == "" {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	r, _ := regexp.Compile(`(\d+).htm`)
	reg := r.FindStringSubmatch(tmp.WirelessShareTpwdQueryResponse.URL)
	if len(reg) < 2 {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]: cannot parse item id")
	}
	tmp.WirelessShareTpwdQueryResponse.ItemID = reg[1]
	return &tmp.WirelessShareTpwdQueryResponse, nil
}

/*{
  "wireless_share_tpwd_query_response": {
    "content": "美国NMN12000增强型9000+基因修复衰老港烟酰胺单核苷酸金達威抗",
    "native_url": "tbopen://m.taobao.com/tbopen/index.html?action=ali.open.nav&module=h5&h5Url=https%3A%2F%2Fa.m.taobao.com%2Fi627075923061.htm%3Fprice%3D3280-28800%26sourceType%3Ditem%26sourceType%3Ditem%26suid%3D5d3ed265-e6cc-4064-8fef-8783a7231cac%26shareUniqueId%3D5353894738%26ut_sk%3D1.WypVVyyFyx8DAJxD5PcJZXYb_21646297_1605168744308.TaoPassword-WeiXin.1%26un%3D343e435bb4718caa4b68d5f77a161062%26share_crt_v%3D1%26spm%3Da211b4.23430885%26sp_tk%3DTWZtRmNsUzdTdUI%3D%26bxsign%3DtcddAn5ztuVKlcbwE0rd1urnFZWTY710gPclZahgUaK3nNSgXvdd-bacfibDkjZkLg8B8JQDxhla0imCdplhBn1ZRD5ABdKIudxk1xducgw_84%26visa%3D13a09278fde22a2e%26disablePopup%3Dtrue%26disableSJ%3D1&appkey=23430885&visa=13a09278fde22a2e",
    "pic_url": "http://gw.alicdn.com/bao/uploaded/i1/2209048632992/O1CN01AJirS91XyNel7itZB_!!0-item_pic.jpg",
    "price": "3280.00",
    "suc": true,
    "thumb_pic_url": "http://gw.alicdn.com/bao/uploaded/i1/2209048632992/O1CN01AJirS91XyNel7itZB_!!0-item_pic.jpg_170x170.jpg",
    "title": "淘口令",
    "url": "https://a.m.taobao.com/i627075923061.htm?price=3280-28800&sourceType=item&sourceType=item&suid=5d3ed265-e6cc-4064-8fef-8783a7231cac&shareUniqueId=5353894738&ut_sk=1.WypVVyyFyx8DAJxD5PcJZXYb_21646297_1605168744308.TaoPassword-WeiXin.1&un=343e435bb4718caa4b68d5f77a161062&share_crt_v=1&spm=a211b4.23430885&sp_tk=TWZtRmNsUzdTdUI=&bxsign=tcddAn5ztuVKlcbwE0rd1urnFZWTY710gPclZahgUaK3nNSgXvdd-bacfibDkjZkLg8B8JQDxhla0imCdplhBn1ZRD5ABdKIudxk1xducgw_84&visa=13a09278fde22a2e&disablePopup=true&disableSJ=1",
    "request_id": "slvl6f7q29k4"
  }
}*/
