package zhimeng

import (
	"encoding/json"
	"fmt"
	"strings"

	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"

	"github.com/tidwall/gjson"
)

var jdState = map[string]int{
	"已付款":  0,
	"已完成":  2,
	"已收货":  2,
	"已结算":  3,
	"订单退款": 4,
}

// promoId一般使用自购ID拼装起来
func JDOrderTrack(appKey, appSecret, webID string, pageNo, pageSize int, sTime, eTime int64, option, dbName string) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2021-04-24 11:32:04").Unix()
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          utils.IntToStr(pageNo),
		"size":       utils.IntToStr(pageSize),
		"start_time": utils.Int64ToStr(sTime), // 这里是秒的时间戳
		"end_time":   utils.Int64ToStr(eTime), // 这里是秒的时间戳
		"domain_id":  webID,
		"dbName":     dbName,
		"is_zyos":    "1",
	}
	resp, err := Send("jd", option, params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	result, err := JdComm(appKey, appSecret, webID, resp, option)
	if err != nil {
		fmt.Println("=======================================", option)
		return nil, logx.Warn(err)
	}
	return result, err
}
func JDOrderSuccessTrack(appKey, appSecret, webID string, ids, dbName string) (*[]md.OrderInfo, error) {
	//sTime = utils.TimeParseStd("2021-04-24 11:32:04").Unix()
	//eTime = sTime + 3600
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"ids":        ids,
		"p":          "1",
		"size":       "100",
		"domain_id":  webID,
		"dbName":     dbName,
		"is_zyos":    "1",
	}
	resp, err := Send("jd", "success_order", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	result, err := JdComm(appKey, appSecret, webID, resp, "")
	if err != nil {
		return nil, logx.Warn(err)
	}
	return result, err
}
func JdComm(appKey, appSecret, webID string, resp []byte, op string) (*[]md.OrderInfo, error) {
	var err error
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    []struct {
			Oid              string `json:"oid"`
			StatusStr        string `json:"status_str"`
			Gid              string `json:"gid"`
			GoodsTitle       string `json:"goods_title"`
			Commission       string `json:"commission"`
			Payment          string `json:"payment"`
			Fukuan           string `json:"fukuan"`
			Shixiao          string `json:"shixiao"`
			Jiesuan          string `json:"jiesuan"`
			Pid              string `json:"pid"`
			CreateTime       string `json:"create_time"`
			PaymentTime      string `json:"payment_time"`
			GoodsImg         string `json:"goods_img"`
			CommissionRate   string `json:"commission_rate"`
			UpdateTime       string `json:"update_time"`
			DomainID         string `json:"domain_id"`
			JiesuanTime      string `json:"jiesuan_time"`
			OrderReceiveTime string `json:"order_receive_time"`
			JdPid            string `json:"jd_pid"`
			DzTime           string `json:"dz_time"`
			ShareApp         string `json:"share_app"`
			ShareAppExtendID string `json:"share_app_extend_id"`
			GoodsInfo        []struct {
				GoodsPrice  string `json:"goods_price"`
				Gid         string `json:"gid"`
				GoodsTitle  string `json:"goods_title"`
				GoodsNum    string `json:"goods_num"`
				SubSideRate string `json:"subSideRate"`
				FinalRate   string `json:"finalRate"`
			} `json:"goods_info"`
			GoodsPrice interface{} `json:"goods_price"`
			GoodsNum   interface{} `json:"goods_num"`
			Type       string      `json:"type"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}

	var (
		res []md.OrderInfo
	)
	for _, v := range tmp.Data {
		var (
			reason    string
			orderType int
		)
		if !strings.Contains(v.Pid, "cloud") && !strings.Contains(v.Pid, "share") && !strings.Contains(v.Pid, "self") && !strings.Contains(v.Pid, "free") {
			continue
		}
		// 分享
		if strings.Contains(v.Pid, "share_") {
			orderType = md.OrderTypeShare
		}
		if strings.Contains(v.Pid, "cloud_") {
			orderType = 10
		}
		// 新人免单
		if strings.Contains(v.Pid, "free_") {
			orderType = md.OrderTypeFree
		}
		if v.StatusStr == "待付款" || v.StatusStr == "订单部分退款" || v.StatusStr == "订单退款" {
			// 跳过待付款的订单
			continue
		}
		state, ok := jdState[v.StatusStr]
		if strings.Contains(v.StatusStr, "无效") {
			state = 4
		}
		if !ok {
			state = md.OrderStateFail
			reason = v.StatusStr
		}
		if utils.StrToInt64(v.PaymentTime) == 0 {
			v.PaymentTime = v.CreateTime
		}
		var confirmAt = utils.StrToInt64(v.OrderReceiveTime)
		if (state == 1 || state == 2) && confirmAt == 0 {
			confirmAt = utils.StrToInt64(v.UpdateTime)
		}
		var JiesuanTime int64 = 0
		if utils.StrToInt64(v.JiesuanTime) > 0 && state != 4 {
			state = md.OrderStatePvdSettle
			JiesuanTime = utils.StrToInt64(v.JiesuanTime)
		}
		pvd := md.PVD_JDOwn
		for _, v1 := range v.GoodsInfo {
			if v1.FinalRate != "100" {
				pvd = md.PVD_JD
			}
		}
		img := ""
		if op == "getneworder" {
			img = getImage(appKey, appSecret, webID, v.Gid, v.GoodsImg)
		}
		res = append(res, md.OrderInfo{
			ConfirmAt:      confirmAt,
			PvdOid:         v.Oid,
			Pvd:            pvd,
			ItemId:         v.Gid,
			ItemNum:        len(v.GoodsInfo),
			ItemPrice:      utils.AnyToFloat64(v.Payment),
			ItemTitle:      v.GoodsTitle,
			ItemLink:       "https://item.m.jd.com/product/" + v.Gid + ".html",
			Thumbnail:      img,
			PaidPrice:      utils.AnyToFloat64(v.Payment),
			OrderType:      orderType, // 默认为自购
			Pid:            v.Pid,     // 推广位ID
			State:          state,
			Commission:     utils.StrToFloat64(v.Commission),
			CommissionRate: utils.StrToFloat64(v.CommissionRate),
			Reason:         reason,
			PayTime:        utils.StrToInt64(v.PaymentTime),
			StartTime:      utils.StrToInt64(v.CreateTime),
			EndTime:        JiesuanTime,
		})
	}
	return &res, nil
}

func getImage(appKey, appSecret, webID string, gid string, img string) string {
	if img != "" {
		return img
	}
	// 为空时去详情拿图片
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"domain_id":  webID,
		"gid":        gid,
	}
	data, err := Send("jd", "getgoodsinfo", params)
	if err != nil {
		return ""
	}
	return gjson.GetBytes(data, "data.goods_img").String()
}

/*
{
    "msg": "订单列表",
    "success": 1,
    "data": [
        {
            "oid": "143791534062",
            "status_str": "已付款",
            "gid": "48461623006",
            "goods_title": "得力USB插头智能有线/无线插座转换器充电魔方多功能家用插排接线板 无线 3组合孔",
            "commission": "0.98",
            "payment": "9.90",
            "fukuan": "1",
            "shixiao": "0",
            "jiesuan": "0",
            "pid": "1_9_jinglianmeng_jd_free_775",
            "create_time": "1611904892",
            "payment_time": "0",
            "goods_img": "",
            "commission_rate": "10.00",
            "update_time": "1611904962",
            "domain_id": "9",
            "jiesuan_time": "0",
            "jd_pid": "3000201681",
            "dz_time": "0",
            "share_app": "0",
            "share_app_extend_id": "0",
            "goods_info": [
                {
                    "goods_price": "16.9",
                    "gid": "48461623006",
                    "goods_title": "得力USB插头智能有线/无线插座转换器充电魔方多功能家用插排接线板 无线 3组合孔",
                    "goods_num": "1"
                }
            ],
            "goods_price": null,
            "goods_num": null,
            "type": "default"
        }
    ]
}

*/
