package zhimeng

import (
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
)

func KuaiShouDetail(tikTokArgs map[string]interface{}) (*TikTokDetailData, error) {
	args := map[string]interface{}{
		"appkey":     tikTokArgs["appkey"],
		"secret_key": tikTokArgs["secret_key"],
		"id":         tikTokArgs["gid"],
	}
	resp, err := Send("kuaishou", "getGoodsDetail", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data TikTokDetailData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}
func KuaiShouMoreDetail(tikTokArgs map[string]interface{}) (map[string]TikTokDetailData, error) {
	args := map[string]interface{}{
		"appkey":     tikTokArgs["appkey"],
		"secret_key": tikTokArgs["secret_key"],
		"id":         tikTokArgs["gid"],
	}
	resp, err := Send("kuaishou", "getMoreGoodsDetail", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data map[string]TikTokDetailData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	data := make(map[string]TikTokDetailData)
	if tmp.Data != nil {
		data = tmp.Data
	}
	return data, nil
}

func KuaiShouKlGoods(tikTokArgs map[string]interface{}) (*TikTokDetailData, error) {
	args := map[string]interface{}{
		"appkey":     tikTokArgs["appkey"],
		"secret_key": tikTokArgs["secret_key"],
		"content":    tikTokArgs["content"],
	}
	resp, err := Send("kuaishou", "getKlGoods", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data TikTokDetailData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}
func KuaiShouGoods(tikTokArgs map[string]interface{}) (*[]TikTokData, error) {
	args := map[string]interface{}{
		"appkey":        tikTokArgs["appkey"],
		"secret_key":    tikTokArgs["secret_key"],
		"title":         tikTokArgs["keyword"],
		"first_cids":    tikTokArgs["category_id"],
		"price_min":     utils.AnyToString(tikTokArgs["start_price"]),
		"price_max":     utils.AnyToString(tikTokArgs["end_price"]),
		"sell_num_min":  utils.AnyToString(tikTokArgs["start_sales"]),
		"sell_num_max":  utils.AnyToString(tikTokArgs["end_sales"]),
		"sort":          tikTokArgs["sort"],
		"cos_ratio_min": utils.AnyToString(tikTokArgs["start_commission"]),
		"cos_ratio_max": utils.AnyToString(tikTokArgs["end_commission"]),
		"page":          tikTokArgs["p"],
		"page_size":     tikTokArgs["size"],
	}
	for k, v := range args {
		if v == nil || v == "<nil>" {
			args[k] = ""
		}
	}
	resp, err := Send("kuaishou", "getGoods", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []TikTokData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}
