package zhimeng

import (
	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"strconv"
)

//加油列表
func OilList(appKey, appSecret string, args *md.OilRequest) (*[]md.OilList, error) {
	params := map[string]interface{}{
		"appkey":     appKey,
		"secret_key": appSecret,
		"p":          args.P,
		"item_name":  args.ItemName,
		"city_name":  args.CityName,
		"lat":        args.Lat,
		"lng":        args.Lng,
		"sort":       args.Sort,
		"brand_name": args.BrandName,
		"mobile":     args.Mobile,
	}
	resp, err := Send("xiaoju", "oil_list", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Msg     string       `json:"msg"`
		Success int          `json:"success"`
		Data    []md.OilList `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if len(tmp.Data) == 0 {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	return &tmp.Data, nil
}

//加油详情
func OilDetail(appKey, appSecret string, args *md.OilRequest, user *md.User) (*md.OilDetail, error) {
	lat := args.Lat
	lng := args.Lng
	params := map[string]interface{}{
		"appkey":       appKey,
		"secret_key":   appSecret,
		"item_name":    args.ItemName,
		"lat":          args.Lat,
		"lng":          args.Lng,
		"store_id":     args.Id,
		"user_id":      utils.IntToStr(user.Info.Uid),
		"mobile":       user.Info.Phone,
		"phone":        user.Info.Phone,
		"nickname":     user.Info.Nickname,
		"zuid":         strconv.Itoa(user.Info.Uid),
		"entrance":     "gasoline/home",
		"avatar":       user.Profile.AvatarUrl,
		"is_zyos":      "1",
		"is_return_jh": args.IsReturnJh,
	}
	if utils.StrToFloat64(lat) > 0 && utils.StrToFloat64(lng) > 0 {
		//args["entrance"] = fmt.Sprintf("gasoline/home?type=%s&lat=%f&lng=%f", "normal", lat, lng)
		params["entrance"] = fmt.Sprintf("gasoline/home?type=%s&lat=%s&lng=%s", "", lat, lng)
	}
	resp, err := Send("xiaoju", "oil_url", params)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    struct {
			Link string `json:"link"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("[resp]: " + string(resp) + ", [err]:" + err.Error())
	}
	if tmp.Data.Link == "" {
		return nil, e.NewErrCode(e.ERR_NO_DATA)
	}
	return &md.OilDetail{Url: tmp.Data.Link}, nil
}
