package zhimeng

import (
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"github.com/tidwall/gjson"
)

func GetBrand(ak, sk string) ([]*md.PrivilegeCardBrand, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
	}
	resp, err := Send("hccard_list", "get_cate", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	//fmt.Println(string(resp))
	var tmp struct {
		Data []*md.PrivilegeCardBrand `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	return tmp.Data, nil
}

func GetGoods(ak, sk string) ([]*md.PrivilegeCardGoods, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
	}
	resp, err := Send("hccard_list", "get_goods", args)
	if err != nil {
		return nil, err
	}
	//fmt.Println(string(resp))
	var tmp struct {
		Data []*md.PrivilegeCardGoods `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	return tmp.Data, nil
}

func GetMoney(ak, sk string) (map[string]string, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
	}
	resp, err := Send("hccard_list", "show_money", args)
	if err != nil {
		return nil, err
	}
	//fmt.Println(string(resp))
	var tmp struct {
		Data struct {
			Money         string `json:"money"`
			PlatformMoney string `json:"platform_money"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn(err)
	}
	return map[string]string{"m": tmp.Data.Money, "pM": tmp.Data.PlatformMoney}, nil
}
func GetGoodsInfo(ak, sk string, specId int) (map[string]string, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
		"id":         utils.IntToStr(specId),
	}
	resp, err := Send("hccard_list", "get_goods_info2", args)
	if err != nil {
		return nil, err
	}
	//fmt.Println(string(resp))
	var tmp struct {
		Data struct {
			GID           string `json:"g_id"`
			ID            string `json:"id"`
			IsDelete      string `json:"is_delete"`
			LID           string `json:"l_id"`
			OfficialPrice string `json:"official_price"`
			PlatPrice     string `json:"plat_price"`
			SpecID        string `json:"spec_id"`
			SpecImg       string `json:"spec_img"`
			Times         string `json:"times"`
			Title         string `json:"title"`
			Type          string `json:"type"`
			UpdateTime    string `json:"update_time"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn(err)
	}
	return map[string]string{"is_delete": tmp.Data.IsDelete}, nil
}

func BuyGoods(ak, sk, account, goodId, amount, uid, oid, orderId, idcard string) (bool, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
		"id":         goodId,  // 商品ID 智盟的商品IsvcD
		"account":    account, // 直冲账号
		"buy_num":    amount,  // 数量
		"zuid":       uid,     // 用户ID
		"l_id":       oid,     // 订单ID
		"oid":        orderId, // 订单号
		"idcard":     idcard,
	}
	resp, err := Send("hccard_list", "pay", args)
	if err != nil {
		// 需要退款
		return false, err
	}
	//fmt.Println(string(resp))
	//return string(resp)
	var tmp struct {
		Msg     string `json:"msg"`
		Success int    `json:"success"`
		Data    struct {
			IsToreturn string `json:"is_toreturn"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return false, logx.Warn(err)
	}
	if tmp.Data.IsToreturn == "1" {
		// 需要退款
		return false, nil
	}

	return true, nil
}

func GetOrder(ak, sk, oid, orderId string) (string, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
		"l_id":       oid,     // 订单ID
		"oid":        orderId, // 订单号
	}
	resp, err := Send("hccard_list", "show_order", args)
	if err != nil {
		return "", err
	}
	r := string(resp)
	fmt.Println("权益卡购买", r)
	returnFlag := gjson.Get(r, "data.is_toreturn").String()
	fmt.Println("returnFlag", returnFlag)
	if returnFlag == "1" {
		// need refund
		return "", nil
	}
	data := gjson.Get(r, "data.data").String()
	return data, nil
}
func GetOrderInfo(ak, sk, oid, orderId string) (string, error) {
	args := map[string]interface{}{
		"appkey":     ak,
		"secret_key": sk,
		"l_id":       oid,     // 订单ID
		"oid":        orderId, // 订单号
	}
	resp, err := Send("hccard_list", "show_order_info", args)
	if err != nil {
		return "", err
	}
	r := string(resp)
	fmt.Println("权益卡购买", r)
	data := gjson.Get(r, "data.data").String()
	return data, nil
}
