package zhimeng

import (
	"applet/app/md"
	"applet/app/utils/logx"
	"encoding/json"
	"fmt"
	"github.com/syyongx/php2go"
	"strconv"
)

// 转链后, 可以通过链接拼装&relationId(int)或者&externalId(string)(二选一)进行订单获取
func TaobaoConvertUrl(appkey, appsecret, itemId, tbPid, tbSid string, tbExternalId, tbAppKey string) (*md.ConvertedUrls, error) {
	args := map[string]interface{}{
		"appkey":          appkey,
		"secret_key":      appsecret,
		"gid":             itemId,
		"new_external_id": tbExternalId, //自定义参数 只有要生成备案链接用
		"pid":             tbPid,
		"tb_app_key":      tbAppKey,
		"session":         tbSid, //tbSid
	}
	fmt.Println(args)
	if php2go.IsNumeric(itemId) {
		args["biz_scene_id"] = "2"
	}
	resp, err := Send("agent_tb", "url", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int `json:"success"`
		Data    struct {
			HaveCoupon        string `json:"have_coupon"`
			URL               string `json:"url"`
			ItemUrl           string `json:"item_url"`
			MaxCommissionRate string `json:"max_commission_rate"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	var hasCoupon bool
	if tmp.Data.HaveCoupon == "1" {
		hasCoupon = true
	}
	return &md.ConvertedUrls{
		ItemId:         itemId,
		URL:            tmp.Data.URL,
		ItemUrl:        tmp.Data.ItemUrl,
		ShortenURL:     tmp.Data.URL,
		NoOpenAppURL:   tmp.Data.URL,
		AppURL:         tmp.Data.URL,
		HasCoupon:      hasCoupon,
		CommissionRate: tmp.Data.MaxCommissionRate,
	}, nil
}

// 活动转链
func TaobaoActivityConvertUrl(appkey, appsecret, tbPid, tbSid, promotion_scene_id string, relationId int64, tbAppKey string) (*md.OfficialRecommendUrl, error) {
	args := map[string]interface{}{
		"appkey":             appkey,
		"secret_key":         appsecret,
		"promotion_scene_id": promotion_scene_id, //活动id
		"pid":                tbPid,
		"session":            tbSid, //tbSid 这里传了, 智盟反而报错
		"tb_app_key":         tbAppKey,
	}
	if relationId > 0 {
		args["relation_id"] = strconv.FormatInt(relationId, 10)
	}
	resp, err := Send("agent_tb", "new_activity_urls", args)

	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Success int `json:"success"`
		Data    struct {
			UrlList struct {
				WxQrcodeUrl       string `json:"wx_qrcode_url"`
				WxMiniprogramPath string `json:"wx_miniprogram_path"`
				Url               string `json:"click_url"`
				ShortClickurl     string `json:"short_click_url"`
			} `json:"url"`
		} `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: + " + string(resp) + " + err:" + err.Error())
	}
	if tmp.Data.UrlList.ShortClickurl != "" {
		tmp.Data.UrlList.Url = tmp.Data.UrlList.ShortClickurl
	}
	return &md.OfficialRecommendUrl{
		WxQrcodeUrl:       tmp.Data.UrlList.WxQrcodeUrl,
		WxMiniprogramPath: tmp.Data.UrlList.WxMiniprogramPath,
		Url:               tmp.Data.UrlList.Url,
	}, nil
}
