package zhimeng

import (
	"applet/app/md"
	"applet/app/utils/logx"
	"encoding/json"
)

type TikTokLiveArgs struct {
	Keyword             string
	PageSize            string
	Sort                string
	StartPrice          string
	EndPrice            string
	StartCommissionRate string
	EndCommissionRate   string
	Page                string
	CategoryId          string
	StartSales          string
	EndSales            string
}
type TikTokLiveData struct {
	LiveId    string                `json:"live_id"`
	Pvd       string                `json:"pvd"`
	Name      string                `json:"name"`
	HeadImg   string                `json:"head_img"`
	FansNum   string                `json:"fans_num"`
	GoodsNum  string                `json:"goods_count"`
	Label     []string              `json:"label"`
	Ext       string                `json:"ext"`
	GoodsList []TikTokLiveGoodsData `json:"goods_list"`
}
type TikTokLiveOneData struct {
	LiveId          string `json:"live_id"`
	Pvd             string `json:"pvd"`
	Name            string `json:"name"`
	HeadImg         string `json:"head_img"`
	FansNum         string `json:"fans_num"`
	GoodsNum        string `json:"goods_count"`
	AuthorLevel     string `json:"author_level"`
	ProductCategory string `json:"product_category"`
	AverageGmv      string `json:"average_gmv"`
}

type TikTokLiveGoodsData struct {
	Commission string        `json:"commission"`
	CostPrice  string        `json:"cost_price"`
	GoodsID    string        `json:"goods_id"`
	GoodsImg   string        `json:"goods_img"`
	GoodsTitle string        `json:"goods_title"`
	Price      string        `json:"price"`
	CoinList   []md.CoinList `json:"coin_list"`
}

type TikTokLiveGoodsWebData struct {
	CostPrice  string        `json:"cost_price"`
	GoodsID    string        `json:"goods_id"`
	GoodsImg   string        `json:"goods_img"`
	GoodsTitle string        `json:"goods_title"`
	Price      string        `json:"price"`
	CoinList   []md.CoinList `json:"coin_list"`
}

func TikTokLive(tikTokArgs map[string]interface{}) (*[]TikTokLiveData, error) {
	args := map[string]interface{}{
		"appkey":     tikTokArgs["appkey"],
		"secret_key": tikTokArgs["secret_key"],
		"page":       tikTokArgs["p"],
		"page_size":  tikTokArgs["size"],
		"keyword":    tikTokArgs["keyword"],
	}
	for k, v := range args {
		if v == nil || v == "<nil>" {
			args[k] = ""
		}
	}
	resp, err := Send("tik_tok_live", "getLive", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []TikTokLiveData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}

func TikTokLiveGoods(tikTokArgs map[string]interface{}) (*[]TikTokData, error) {
	args := map[string]interface{}{
		"appkey":          tikTokArgs["appkey"],
		"secret_key":      tikTokArgs["secret_key"],
		"author_buyin_id": tikTokArgs["author_buyin_id"],
	}
	for k, v := range args {
		if v == nil || v == "<nil>" {
			args[k] = ""
		}
	}
	resp, err := Send("tik_tok_live", "getLiveGoods", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data []TikTokData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}

func TikTokLiveOne(tikTokArgs map[string]interface{}) (*TikTokLiveOneData, error) {
	args := map[string]interface{}{
		"appkey":      tikTokArgs["appkey"],
		"secret_key":  tikTokArgs["secret_key"],
		"author_info": tikTokArgs["author_info"],
	}
	for k, v := range args {
		if v == nil || v == "<nil>" {
			args[k] = ""
		}
	}
	resp, err := Send("tik_tok_live", "getOneLive", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data TikTokLiveOneData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}

func TikTokLiveUrl(tikTokArgs map[string]interface{}) (*TikTokUrlData, error) {
	args := map[string]interface{}{
		"appkey":        tikTokArgs["appkey"],
		"secret_key":    tikTokArgs["secret_key"],
		"open_id":       tikTokArgs["live_id"],
		"product_id":    tikTokArgs["product_id"],
		"external_info": tikTokArgs["external_info"],
	}
	for k, v := range args {
		if v == nil || v == "<nil>" {
			args[k] = ""
		}
	}
	resp, err := Send("tik_tok_live", "getUrls", args)
	if err != nil {
		return nil, logx.Warn(err)
	}
	var tmp struct {
		Data TikTokUrlData `json:"data"`
	}
	if err = json.Unmarshal(resp, &tmp); err != nil {
		return nil, logx.Warn("resp: " + string(resp) + ", err:" + err.Error())
	}
	return &tmp.Data, nil
}
