package svc

import (
	"applet/app/cfg"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/md"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"fmt"
	"xorm.io/xorm"
)

func CommCheckFastReturn(eg *xorm.Engine, uid int, mid string) (int, *model.UserAlipayInfo) {
	fastReturnOnoff, _ := db.SysCfgGetOne(eg, "fast_return_onoff")
	if fastReturnOnoff == nil {
		return 0, nil
	}
	info := db.GetUserAlipayInfo(eg, uid)
	if info == nil {
		return 0, nil
	}
	if info.IsSign == 0 || info.IsOpen == 0 || info.CreditAgreementId == "" || fastReturnOnoff.Val != "1" {
		return 0, nil
	}

	return 1, info
}
func FastReturnReduceOrder(eg *xorm.Engine, uid int, oid, mid string) {
	var orderData model.OrdList
	b, err2 := eg.Where("ord_id=?", oid).Get(&orderData)
	if b == false || err2 != nil {
		return
	}
	if orderData.State != 4 || orderData.IsWaitReduce > 0 || orderData.FastSignId == "" {
		return
	}
	var data model.OrdListRelate
	get, err := eg.Where("uid=? and oid=? and level=?", uid, oid, 0).Get(&data)
	if get == false || err != nil {
		return
	}
	amount := int(data.Amount * 100)
	if amount <= 0 {
		return
	}
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return
	}
	mutexKey := fmt.Sprintf("%s:FastReturnReduceOrder:%s", mid, oid)
	withdrawAvailable, err := cache.Do("SET", mutexKey, 1, "EX", 120, "NX")
	if err != nil {
		return
	}
	if withdrawAvailable != "OK" {
		return
	}
	prd := "false"
	if cfg.Prd {
		prd = "true"
	}
	arg := md.ZhiosFatReturnOrderPay{
		Uid:  utils.IntToStr(uid),
		Mid:  mid,
		Oid:  oid,
		Name: "抖省",
		Prd:  prd,
	}
	defer ch.Release()
	ch.Publish(md.FastReturnOrder, utils.SerializeStr(arg), md.FastReturnOrderRoutKeyForOrderRefund)
	return
}
func FastReturnReduceOrderSecond(eg *xorm.Engine, uid int, oid, mid string) {
	var orderData model.OrdList
	b, err2 := eg.Where("ord_id=?", oid).Get(&orderData)
	if b == false || err2 != nil {
		return
	}
	if orderData.IsWaitReduce > 0 || orderData.FastSignId == "" {
		return
	}
	var data model.OrdListRelate
	get, err := eg.Where("uid=? and oid=? and level=?", uid, oid, 0).Get(&data)
	if get == false || err != nil {
		return
	}
	amount := int(data.Amount * 100)
	if amount <= 0 {
		return
	}
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return
	}
	mutexKey := fmt.Sprintf("%s:FastReturnReduceOrder:%s", mid, oid)
	withdrawAvailable, err := cache.Do("SET", mutexKey, 1, "EX", 120, "NX")
	if err != nil {
		return
	}
	if withdrawAvailable != "OK" {
		return
	}
	prd := "false"
	if cfg.Prd {
		prd = "true"
	}
	arg := md.ZhiosFatReturnOrderPay{
		Uid:    utils.IntToStr(uid),
		Mid:    mid,
		Oid:    oid,
		Name:   "抖省",
		Prd:    prd,
		Amount: amount,
	}
	defer ch.Release()
	ch.Publish(md.FastReturnOrder, utils.SerializeStr(arg), md.FastReturnOrderRoutKeyForOrderRefundSecond)
	return
}
func FastReturnFinishOrder(eg *xorm.Engine, uid int, oid, mid string) {
	var orderData model.OrdList
	b, err2 := eg.Where("ord_id=?", oid).Get(&orderData)
	if b == false || err2 != nil {
		return
	}
	if orderData.FastSignId == "" {
		return
	}
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return
	}
	mutexKey := fmt.Sprintf("%s:FastReturnFinishOrder:%s", mid, oid)
	withdrawAvailable, err := cache.Do("SET", mutexKey, 1, "EX", 120, "NX")
	if err != nil {
		return
	}
	if withdrawAvailable != "OK" {
		return
	}
	prd := "false"
	if cfg.Prd {
		prd = "true"
	}
	arg := md.ZhiosFatReturnOrderPay{
		Uid:  utils.IntToStr(uid),
		Mid:  mid,
		Oid:  oid,
		Name: "抖省",
		Prd:  prd,
	}
	defer ch.Release()
	ch.Publish(md.FastReturnOrder, utils.SerializeStr(arg), md.FastReturnOrderRoutKeyForOrderSuccess)
	return
}
func FastReturnCreateOrder(eg *xorm.Engine, uid int, oid, mid string) {
	isFastReturn, info := CommCheckFastReturn(eg, uid, mid)
	if isFastReturn == 0 {
		return
	}

	var orderData model.OrdList
	b, err2 := eg.Where("ord_id=?", oid).Get(&orderData)
	if b == false || err2 != nil {
		return
	}
	if orderData.SettleAt > 0 || orderData.IsTransfer == 1 || orderData.IsFastSign == 1 || orderData.State == 4 || orderData.IsWaitReduce > 0 {
		return
	}
	var data model.OrdListRelate
	get, err := eg.Where("uid=? and oid=? and level=?", uid, oid, 0).Get(&data)
	if get == false || err != nil {
		return
	}
	amount := int(data.Amount * 100)
	if amount <= 0 || data.Amount < 0.1 {
		return
	}
	// 获取所有抽成方案的计划
	allRewardPlan, err := db.DbsPlanRewardByPvds(eg)
	if err != nil || allRewardPlan == nil {
		return
	}
	isCanReturn := 0
	for _, v := range allRewardPlan {
		if v.Pvd == orderData.Pvd {
			isCanReturn = v.IsFastReturn
		}
	}
	if isCanReturn == 0 {
		return
	}
	mutexKey := fmt.Sprintf("%s:FastReturnCreateOrder:%s", mid, oid)
	withdrawAvailable, err := cache.Do("SET", mutexKey, 1, "EX", 120, "NX")
	if err != nil {
		return
	}
	if withdrawAvailable != "OK" {
		return
	}

	money := float64(amount) / 100
	fastReturnMoney := db.SysCfgFindWithDb(eg, mid, "fast_return_money")
	if utils.StrToFloat64(info.Money)+money > utils.StrToFloat64(fastReturnMoney["fast_return_money"]) {
		return
	}
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return
	}
	prd := "false"
	if cfg.Prd {
		prd = "true"
	}
	arg := md.ZhiosFatReturnOrderPay{
		Uid:  utils.IntToStr(uid),
		Mid:  mid,
		Oid:  oid,
		Name: "抖省",
		Prd:  prd,
	}
	defer ch.Release()
	ch.Publish(md.FastReturnOrder, utils.SerializeStr(arg), md.FastReturnOrderRoutKeyForOrderPay)
	return
}
