package svc

import (
	"applet/app/db"
	"applet/app/lib/plan"
	"applet/app/md"
	"applet/app/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/lib/comm_plan"
	"errors"
	"fmt"
	"strings"
	"xorm.io/xorm"
)

func getBili(eg *xorm.Engine, masterId, types string) float64 {
	//读取佣金与 区块币比例
	biliList := db.SysCfgGetWithDb(eg, masterId, types)
	var bili float64 = 1
	if biliList != "" {
		list := strings.Split(biliList, ":")
		if len(list) == 2 {
			bili = utils.StrToFloat64(list[0]) / utils.StrToFloat64(list[1])
			bili = utils.FloatFormat(bili, 6)
		}
	}
	return bili
}

// 根据用户计算对应的手续费
//ownbuyReturnType 0返利 1不返利
//pvd 只是为了判断是会员升级的订单 可不传
func CalcCommission(uid, level, ownbuyReturnType int, fee float64, isShare bool, opt *plan.PlanOpt, userRelationShip *[]md.UserRelation, pvd string, new_level int, masterId string) (pvdFee, sysFee, subsidyFee float64, lvUser *plan.LvUser, err error, regionFee, globalFee float64) {
	pvdFee = fee * opt.PvdRate // 供应商联盟比例
	sysFee = fee * opt.SysRate // 平台比例
	eg := db.DBs[masterId]
	isRegionAgentOpen := db.SysCfgGetWithDb(eg, masterId, "regional_agent_is_open")
	if isRegionAgentOpen == "1" {
		regionFee = fee * opt.RegionRate //区域代理
	}

	ms, _ := db.CapitalPoolByIsUse(eg)
	if ms != nil {
		globalFee = fee * opt.GlobalRate //全球分红比例
	}
	pushHandFee := fee * opt.PushHandRate
	merchantFee := fee * opt.MerchantRate
	// 剩余的余额
	fee = float64(int64(fee*1e4)-int64(pvdFee*1e4)-int64(sysFee*1e4)-int64(regionFee*1e4)-int64(globalFee*1e4)-int64(pushHandFee*1e4)-int64(merchantFee*1e4)) / 1e4
	// 计算自购补贴比例
	subsidyFee = 0
	//如果没登录，要找出权重最低的那个
	if uid == 0 {
		lvList, err := db.UserLevelInIDescByWeightLow(eg)
		if err != nil {
			fmt.Println(err)
		}
		if lvList != nil {
			for _, v := range lvList {
				level = v.Id
			}
		}
	}
	// 判断是否存在
	if _, ok := opt.UserRate[level]; !ok {
		return 0, 0, 0, nil, errors.New("opt.UserRate[level] is nil uid=" + utils.IntToStr(uid)), regionFee, globalFee
	}

	if opt.UserRate[level].SubsidyEnable == 1 { //额外补贴给消费者
		if isShare && opt.UserRate[level].SubsidyMode == 0 {
			subsidyFee = fee * opt.UserRate[level].SubsidyShareRate
		} else if isShare && opt.UserRate[level].SubsidyMode == 1 {
			subsidyFee = sysFee * opt.UserRate[level].SubsidyShareRate
		} else if !isShare && opt.UserRate[level].SubsidyMode == 0 {
			subsidyFee = fee * opt.UserRate[level].SubsidySelfRate
		} else {
			subsidyFee = sysFee * opt.UserRate[level].SubsidySelfRate
		}
	}
	subsidyFee = utils.FloatFormat(subsidyFee, 6)
	//读取佣金与积分 比例
	var commissionIntegralBili = getBili(eg, masterId, "commission_integral_bili")
	var commissionBlockIconsBili = getBili(eg, masterId, "commission_block_icons_bili")
	opt.IntegralBili = commissionIntegralBili
	opt.BlockIconsBili = commissionBlockIconsBili
	// 获取用户的关系网
	//userRelationShip, err := UserRelativeNetwork(eg, uid)
	// 如果获取用户关系失败或者佣金方案不存在, 那么只计算当前用户的佣金, 平台佣金, 以及供应商方的佣金
	if _, ok := plan.Fn[opt.Mode]; !ok || userRelationShip == nil || len(*userRelationShip) == 0 {
		commission, _ := plan.BuyBili(level, ownbuyReturnType, 0, "own", fee, opt)
		return pvdFee, sysFee, subsidyFee, &plan.LvUser{Uid: uid, Lv: level, Profit: commission, Diff: 0}, nil, regionFee, globalFee
	}
	//查出所有等级替换权重
	levelList, _ := db.UserLevlEgAll(eg)
	levelWeight := 0
	for _, v1 := range levelList {
		levelId := level
		if v1.Id == levelId {
			levelWeight = v1.LevelWeight
		}
	}
	lvUser = &plan.LvUser{Uid: uid, Lv: level, Diff: 0, NewLv: new_level, LevelWeight: levelWeight, OwnbuyReturnType: ownbuyReturnType}
	var node = lvUser
	//关系链处理
	for _, v := range *userRelationShip {
		for _, v1 := range levelList {
			levelId := v.Level
			if v1.Id == levelId {
				levelWeight = v1.LevelWeight
			}
		}
		node.ParentUser = &plan.LvUser{Uid: v.Uid, Lv: v.Level, Diff: v.Diff, LevelWeight: levelWeight}
		node = node.ParentUser
	}
	if err := plan.Fn[opt.Mode](opt.UserRate, fee, lvUser, pvd, sysFee, opt); err != nil {
		fmt.Println("方案有问题1")
		return 0, 0, 0, nil, err, regionFee, globalFee
	}

	return pvdFee, sysFee, subsidyFee, lvUser, nil, regionFee, globalFee
}

func CalcLvUserFee(lvUser *comm_plan.LvUser) map[int]float64 {
	fee := map[int]float64{}
	if lvUser == nil {
		return fee
	}
	if lvUser.ParentUser == nil {
		fee[lvUser.Uid] = lvUser.Profit
	}
	for lvUser.ParentUser != nil {
		fee[lvUser.Uid] = lvUser.Profit
		lvUser = lvUser.ParentUser
	}
	return fee
}

func SerializeLvUser(lvUser map[int]float64) string {
	s := ""
	for k, v := range lvUser {
		s += "|" + utils.IntToStr(k) + ":" + utils.Float64ToStr(v)
	}
	return s
}
