package svc

import (
	"applet/app/cfg"
	"applet/app/db"
	"applet/app/md"
	"applet/app/utils/cache"
	"applet/app/utils/logx"
	"crypto/tls"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"strings"
	"time"
	"xorm.io/xorm"
)

//检测优惠券扣钱
func ApiToReduceCoupon(masterId string, req *md.CouponCheck) int {
	logx.Info(req)

	//检测优惠券功能是否开启
	bools := CouponOpen(db.DBs[masterId])
	if bools == false {
		return 1
	}
	// 加锁 防止并发提取
	mutexKey := fmt.Sprintf("%s:ReduceCoupon:%s", masterId, req.Uid)
	withdrawAvailable, err := cache.Do("SET", mutexKey, 1, "EX", 1, "NX")
	if err != nil {
		return 2
	}
	if withdrawAvailable != "OK" {
		return 2
	}
	var host string
	host = cfg.AppComm.URL + "/api/v1/inComm/Coupon/reduce"
	tr := &http.Transport{
		MaxIdleConnsPerHost: 200,
		MaxIdleConns:        200,
		MaxConnsPerHost:     200,
		TLSClientConfig:     &tls.Config{InsecureSkipVerify: true},
	}
	client := &http.Client{
		Timeout:   5 * time.Second,
		Transport: tr,
	}
	byte1, _ := json.Marshal(req)
	req1, _ := http.NewRequest("POST", host, strings.NewReader(string(byte1)))
	req1.Header.Set("master_id", masterId)
	req1.Header.Set("Content-Type", "application/json")
	resp, err := (client).Do(req1)
	logx.Info(req)

	logx.Info(resp)
	if err != nil || resp == nil {
		if resp != nil {
			_, _ = io.Copy(ioutil.Discard, resp.Body)
		}
		return 0
	}
	defer resp.Body.Close()
	respByte, _ := ioutil.ReadAll(resp.Body)
	logx.Info(string(respByte))
	if len(respByte) == 0 {
		return 0
	}
	var serverResp map[string]interface{}
	err = json.Unmarshal(respByte, &serverResp)
	logx.Info(serverResp)
	if err != nil || serverResp == nil {
		return 0
	}
	if serverResp["data"] == nil {
		if serverResp["msg"] != "" {
			return 0
		}
		return 0
	}
	if serverResp["data"] == "success" {
		return 1
	}
	return 0
}

func CouponOpen(eg *xorm.Engine) bool {
	//查出基本配置
	baseList, err := db.CommCouponBaseList(eg)
	if err != nil || baseList == nil {
		return false
	}
	if baseList.Id == 0 {
		return false
	}
	//判断有没有开启
	if baseList.IsUse == 0 {
		return false
	}
	//判断有没有这个渠道
	if baseList.BonusType == "" {
		return false
	}
	return true
}
