package svc

import (
	"applet/app/cfg"
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/lib/zhimeng"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_o2o_business.git/o2o"
	db2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/tidwall/gjson"
	"time"
	"xorm.io/xorm"

	"applet/app/md"
	"applet/app/utils/logx"
)

func PddOrderRefundTrack(eg *xorm.Engine, dbName string) {
	var (
		pageNo   int = 1
		pageSize int = 100
	)
	zhim := SysCfgFindByIdsToCache(eg, dbName, md.ZHIMENG_CFG_LIST...)
	if zhim == nil {
		return
	}
	if zhim[md.KEY_CFG_ZM_AK] == "" || zhim[md.KEY_CFG_ZM_SK] == "" {
		return
	}
	pddRefundKeys := SysCfgFindByIdsToCache(eg, dbName, "pdd_refund_p")
	if pddRefundKeys["pdd_refund_p"] != "" && pddRefundKeys["pdd_refund_p"] != "0" {
		pageNo = utils.StrToInt(pddRefundKeys["pdd_refund_p"])
	} else {
		db.DbsSysCfgInserts(eg, "pdd_refund_p", "1")
	}
	littlePocket := db.SysCfgFindWithDb(eg, dbName, "little_pocket_basic_data")
	returnMoneyType := "0"
	if littlePocket != nil && gjson.Get(littlePocket["little_pocket_basic_data"], "return_money_type").String() == "1" {
		returnMoneyType = "1"
	}
	ordData, err := zhimeng.PddOrderTrack(zhim[md.KEY_CFG_ZM_AK], zhim[md.KEY_CFG_ZM_SK], pageNo, pageSize, 0, 0, "refund_order", dbName, "")
	if err != nil {
		return
	}
	//匹配存在的订单
	var allOrdIds []string
	for _, v := range *ordData {
		allOrdIds = append(allOrdIds, v.PvdOid)
	}

	existOrds, _ := db.DbsOrderFindByIds(eg, md.PVD_PDD, allOrdIds)
	if existOrds != nil {
		for _, v := range *existOrds {
			//不是平台结算状态跟没有结算时间其中一个就跳过 或者没用户跳过
			if v.SettleAt == 0 && v.State < 5 || v.Uid == 0 {
				continue
			}
			// 加锁 防止并发退款
			mutexKey := fmt.Sprintf("%s:all_order_refund:%d:pvd:%s", dbName, v.OrdId, v.Pvd)
			available, err := cache.Do("SET", mutexKey, 1, "EX", 5, "NX")
			if err != nil {
				continue
			}
			if available != "OK" {
				continue
			}
			RefundDoing(eg, returnMoneyType, &v, "", dbName)
		}
	}
	if len(*ordData) == pageSize {
		fmt.Println("========退款订单下一页========" + utils.IntToStr(pageNo))
		pageNo++
		db.DbsSysCfgUpdate(eg, "pdd_refund_p", utils.IntToStr(pageNo))
	}

	return
}

//商家分佣退回
func commStoreRefund(eg *xorm.Engine, session *xorm.Session, ordId, pvd string) {
	//商家分佣
	args := map[string]string{
		"oid":   ordId,
		"pvd":   pvd,
		"title": "导购商品退款，扣除分佣",
	}
	if pvd == md.PVD_CARD {
		args["title"] = "权益卡商品退款，扣除分佣"
	}
	bools := o2o.DeductStoreCommission(eg, session, args)
	if bools == false {
		_ = session.Rollback()
		return
	}
	return
}
func commRefund(eg *xorm.Engine, session *xorm.Session, existOrd *model.OrdList, uid int, pvd, itemId string, ol []*model.OrdListRelate, title, mid string) {
	if title == "" {
		title = "退款"
	}

	now := time.Now()
	for _, item := range ol {
		userProfile, err := db.UserProfileFindByIdWithSession(session, item.Uid)
		if userProfile == nil {
			_ = logx.Warnf("ORDER REFUND: ORDER ID: %s, USER ID %s CAN NOT FIND IN  USER PROFILE!!\n", item.Oid, item.Uid)
		}
		if err != nil || userProfile == nil {
			_ = session.Rollback()
			return
		}
		if utils.StrToFloat64(item.AdditionalSubsidy) > 0 && item.Mode != "public_platoon" {
			AdditionalSubsidy := utils.StrToFloat64(item.AdditionalSubsidy)
			// 更新用户余额
			beforeAmount := userProfile.FinValid
			userProfile.FinValid = utils.Float64ToStrByPrec(utils.AnyToFloat64(userProfile.FinValid)-AdditionalSubsidy, 4)
			userProfile.FinTotal = userProfile.FinTotal - utils.StrToFloat32(utils.AnyToString(AdditionalSubsidy))

			affected, err := db.UserProfileUpdateWithSession(session, item.Uid, userProfile, "fin_valid", "fin_total")
			if affected == 0 {
				_ = session.Rollback()
				_ = logx.Warn("ORDER SETTLE: UPDATE USER PROFILE RETURN AFFECTED 0")
				return
			}
			if err != nil {
				_ = session.Rollback()
				return
			}

			// 开始写入流水
			ItemTitle := ""
			var orderAction = 22
			if uid == item.Uid {
				ItemTitle = md.PVD_LIST[pvd] + "订单" + title + "扣除补贴金额"
			} else { // 团队
				ItemTitle = "团队下" + md.PVD_LIST[pvd] + "订单" + title + "扣除补贴金额"
			}
			if err := db.FinUserFlowInsertOneWithSession(
				session,
				&model.FinUserFlow{
					Type:         1,
					Uid:          item.Uid,
					Amount:       utils.AnyToString(AdditionalSubsidy),
					BeforeAmount: beforeAmount,
					AfterAmount:  userProfile.FinValid,
					OrdType:      pvd,
					OrdId:        utils.AnyToString(item.Oid),
					OrdAction:    orderAction,
					OrdDetail:    itemId,
					State:        2,
					OtherId:      item.Id,
					OrdTitle:     ItemTitle,
					OrdTime:      int(now.Unix()),
					CreateAt:     now,
					UpdateAt:     now,
				}); err != nil {
				_ = session.Rollback()
				_ = logx.Warn(err)
				return
			}
		}
		// 佣金为0不处理

		if existOrd.IsFastSign == 1 && existOrd.Uid == item.Uid && item.Level == 0 && existOrd.IsWaitReduce == 0 {
			FastReturnReduceOrder(eg, existOrd.Uid, utils.Int64ToStr(existOrd.OrdId), mid)
		}
		// 更新用户余额
		beforeAmount := userProfile.FinValid
		userProfile.FinValid = utils.Float64ToStrByPrec(utils.AnyToFloat64(userProfile.FinValid)-item.Amount, 4)
		userProfile.FinTotal = userProfile.FinTotal - utils.StrToFloat32(utils.AnyToString(item.Amount))

		_, err = db.UserProfileUpdateWithSession(session, item.Uid, userProfile, "fin_valid", "fin_total")
		//if affected == 0 {
		//	_ = session.Rollback()
		//	_ = logx.Warn("ORDER SETTLE: UPDATE USER PROFILE RETURN AFFECTED 0")
		//	return
		//}
		if err != nil {
			_ = session.Rollback()
			return
		}

		// 开始写入流水
		ItemTitle := ""
		var orderAction = 22
		if uid == item.Uid {
			ItemTitle = md.PVD_LIST[pvd] + "订单" + title + "扣除佣金"
		} else { // 团队
			ItemTitle = "团队下" + md.PVD_LIST[pvd] + "订单" + title + "扣除佣金"
		}
		if err := db.FinUserFlowInsertOneWithSession(
			session,
			&model.FinUserFlow{
				Type:         1,
				Uid:          item.Uid,
				Amount:       utils.AnyToString(item.Amount),
				BeforeAmount: beforeAmount,
				AfterAmount:  userProfile.FinValid,
				OrdType:      pvd,
				OrdId:        utils.AnyToString(item.Oid),
				OrdAction:    orderAction,
				OrdDetail:    itemId,
				State:        2,
				OtherId:      item.Id,
				OrdTitle:     ItemTitle,
				OrdTime:      int(now.Unix()),
				CreateAt:     now,
				UpdateAt:     now,
			}); err != nil {
			_ = session.Rollback()
			_ = logx.Warn(err)
			return
		}
	}

	return
}
func commTikTokRefund(session *xorm.Session, uid int, pvd, itemId string, ol []*model.TikTokTeamOrderRelate, title string) {
	if title == "" {
		title = "退款"
	}

	now := time.Now()
	for _, item := range ol {
		// 佣金为0不处理
		if utils.StrToFloat64(item.Amount) < 0.01 {
			continue
		}
		if item.CoinId == 0 {
			userProfile, err := db.UserProfileFindByIdWithSession(session, item.Uid)
			if userProfile == nil {
				_ = logx.Warnf("ORDER REFUND: ORDER ID: %s, USER ID %s CAN NOT FIND IN  USER PROFILE!!\n", item.Oid, item.Uid)
			}
			if err != nil || userProfile == nil {
				_ = session.Rollback()
				return
			}
			// 更新用户余额
			beforeAmount := userProfile.FinValid
			userProfile.FinValid = utils.Float64ToStrByPrec(utils.AnyToFloat64(userProfile.FinValid)-utils.AnyToFloat64(item.Amount), 4)
			userProfile.FinTotal = userProfile.FinTotal - utils.StrToFloat32(utils.AnyToString(item.Amount))

			affected, err := db.UserProfileUpdateWithSession(session, item.Uid, userProfile, "fin_valid", "fin_total")
			if affected == 0 {
				_ = session.Rollback()
				_ = logx.Warn("ORDER SETTLE: UPDATE USER PROFILE RETURN AFFECTED 0")
				return
			}
			if err != nil {
				_ = session.Rollback()
				return
			}

			// 开始写入流水
			ItemTitle := "抖团额外奖励扣除"
			if err := db.FinUserFlowInsertOneWithSession(
				session,
				&model.FinUserFlow{
					Type:         1,
					Uid:          item.Uid,
					Amount:       utils.AnyToString(item.Amount),
					BeforeAmount: beforeAmount,
					AfterAmount:  userProfile.FinValid,
					OrdType:      pvd,
					OrdId:        utils.AnyToString(item.Oid),
					OrdAction:    22,
					OrdDetail:    itemId,
					State:        2,
					OtherId:      int64(item.Id),
					OrdTitle:     ItemTitle,
					OrdTime:      int(now.Unix()),
					CreateAt:     now,
					UpdateAt:     now,
				}); err != nil {
				_ = session.Rollback()
				_ = logx.Warn(err)
				return
			}

		} else {
			var (
				beforeAmount = "0"
				afterAmount  = "0"
			)
			// 查询用户虚拟币余额表记录，有则更新，无则新增一条记录
			userVirtualAmount, err := db.GetUserVirtualAmountOne(session, item.Uid, item.CoinId)
			if err != nil {
				session.Rollback()
				return
			}

			if userVirtualAmount == nil { // 没有记录则新增一条
				continue
			} else { // 更新
				beforeAmount = userVirtualAmount.Amount
				newAmount := utils.StrToFloat64(userVirtualAmount.Amount) - utils.StrToFloat64(item.Amount)
				userVirtualAmount.Amount = utils.Float64ToStrPrec4(newAmount)
				afterAmount = userVirtualAmount.Amount
				affected, err := session.Where("id = ?", userVirtualAmount.Id).Cols("amount").Update(userVirtualAmount)
				if err != nil {
					_ = logx.Warn(err)
					session.Rollback()
					return
				}
				if affected == 0 {
					_ = logx.Warn("ORDER SETTLE: UPDATE ORDER RETURN AFFECTED 0")
					session.Rollback()
					return
				}
			}
			ItemTitle := "抖团额外奖励扣除"
			// 查询订单商品信息 会员费订单没商品跳过
			// 用户虚拟币流水表新增记录
			var userVirtualCoinFlow = model.UserVirtualCoinFlow{
				Uid:          item.Uid,
				CoinId:       item.CoinId,
				Direction:    2,
				Title:        ItemTitle,
				OrdId:        item.Oid,
				Amout:        item.Amount,
				BeforeAmout:  beforeAmount,
				AfterAmout:   afterAmount,
				SysFee:       "0",
				CreateTime:   time.Now(),
				TransferType: 55,
			}
			has, errs := db.InsertCommWithSession(
				session, &userVirtualCoinFlow)
			if errs != nil || has == 0 {
				_ = logx.Warn(err)
				session.Rollback()
				return
			}
			return
		}
	}
	return
}
func commVirtualCoinRefund(session *xorm.Session, uid int, pvd, itemId string, ol []*model.VirtualCoinRelate, title string) {
	if title == "" {
		title = "退款"
	}

	for _, item := range ol {
		// 佣金为0不处理
		if utils.StrToFloat64(item.Amount) < 0.01 {
			continue
		}
		var (
			beforeAmount = "0"
			afterAmount  = "0"
		)
		// 查询用户虚拟币余额表记录，有则更新，无则新增一条记录
		userVirtualAmount, err := db.GetUserVirtualAmountOne(session, item.Uid, item.CoinId)
		if err != nil {
			session.Rollback()
			return
		}
		if userVirtualAmount == nil { // 没有记录则新增一条
			continue
		} else { // 更新
			beforeAmount = userVirtualAmount.Amount
			newAmount := utils.StrToFloat64(userVirtualAmount.Amount) - utils.StrToFloat64(item.Amount)
			userVirtualAmount.Amount = utils.Float64ToStrPrec4(newAmount)
			afterAmount = userVirtualAmount.Amount
			affected, err := session.Where("id = ?", userVirtualAmount.Id).Cols("amount").Update(userVirtualAmount)
			if err != nil {
				_ = logx.Warn(err)
				session.Rollback()
				return
			}
			if affected == 0 {
				_ = logx.Warn("ORDER SETTLE: UPDATE ORDER RETURN AFFECTED 0")
				session.Rollback()
				return
			}
		}
		ItemTitle := ""
		if uid == item.Uid {
			ItemTitle = md.PVD_LIST[pvd] + "订单" + title + "扣除佣金"
		} else { // 团队
			ItemTitle = "团队下" + md.PVD_LIST[pvd] + "订单" + title + "扣除佣金"
		}
		// 查询订单商品信息 会员费订单没商品跳过
		// 用户虚拟币流水表新增记录
		var userVirtualCoinFlow = model.UserVirtualCoinFlow{
			Uid:          item.Uid,
			CoinId:       item.CoinId,
			Direction:    2,
			Title:        ItemTitle,
			OrdId:        utils.Int64ToStr(item.Oid),
			Amout:        item.Amount,
			BeforeAmout:  beforeAmount,
			AfterAmout:   afterAmount,
			SysFee:       "0",
			CreateTime:   time.Now(),
			TransferType: 55,
		}
		has, errs := db.InsertCommWithSession(
			session, &userVirtualCoinFlow)
		if errs != nil || has == 0 {
			_ = logx.Warn(err)
			session.Rollback()
			return
		}
	}
	return
}
func commAreaRefund(session *xorm.Session, uid int, oid, pvd, itemId, title string) {
	if title == "" {
		title = "退款"
	}
	ol, err := db.AgentOrderListRelateByOidAndPvdWithSession(session, oid, pvd)
	if err != nil || ol == nil {
		return
	}
	now := time.Now()
	for _, item := range ol {
		// 佣金为0不处理
		if item.DivideAmount < 0.01 {
			continue
		}
		userProfile, err := db.UserProfileFindByIdWithSession(session, item.Uid)
		if userProfile == nil {
			_ = logx.Warnf("ORDER REFUND: ORDER ID: %s, USER ID %s CAN NOT FIND IN  USER PROFILE!!\n", item.BelongOrdId, item.Uid)
		}
		if err != nil || userProfile == nil {
			_ = session.Rollback()
			return
		}
		// 更新用户余额
		beforeAmount := userProfile.FinValid
		userProfile.FinValid = utils.AnyToString(utils.AnyToFloat64(userProfile.FinValid) - item.DivideAmount)
		userProfile.FinTotal = userProfile.FinTotal - utils.StrToFloat32(utils.AnyToString(item.DivideAmount))

		affected, err := db.UserProfileUpdateWithSession(session, item.Uid, userProfile, "fin_valid", "fin_total")
		if affected == 0 {
			_ = session.Rollback()
			_ = logx.Warn("ORDER SETTLE: UPDATE USER PROFILE RETURN AFFECTED 0")
			return
		}
		if err != nil {
			_ = session.Rollback()
			return
		}

		// 开始写入流水
		ItemTitle := ""
		var orderAction = 22
		if uid == item.Uid {
			ItemTitle = "导购订单" + title + "扣除区域代理佣金"
		} else { // 团队
			ItemTitle = "团队下导购订单" + title + "扣除区域代理佣金"
		}
		if err := db.FinUserFlowInsertOneWithSession(
			session,
			&model.FinUserFlow{
				Type:         1,
				Uid:          item.Uid,
				Amount:       utils.AnyToString(item.DivideAmount),
				BeforeAmount: beforeAmount,
				AfterAmount:  userProfile.FinValid,
				OrdType:      pvd,
				OrdId:        utils.AnyToString(item.BelongOrdId),
				OrdAction:    orderAction,
				OrdDetail:    itemId,
				State:        2,
				OtherId:      item.Id,
				OrdTitle:     ItemTitle,
				OrdTime:      int(now.Unix()),
				CreateAt:     now,
				UpdateAt:     now,
			}); err != nil {
			_ = session.Rollback()
			_ = logx.Warn(err)
			return
		}
	}
	db.AgentOrderListRelateUpdateWithSession(session, oid, pvd, 4)
	return
}

//处理退款判断
func RefundDoing(eg *xorm.Engine, returnMoneyType string, order *model.OrdList, title, dbName string) {

	// 查询数据 怕订单重复加上渠道
	ol, err := db.OrderRelateListByOid(eg, order.OrdId, order.Pvd)
	if err != nil {
		_ = logx.Warn(err)
		return
	}
	tikTokOl, err := db.TikTokTeamOrderOrderRelateListByOid(eg, order.OrdId, order.Pvd)
	if err != nil {
		_ = logx.Warn(err)
		return
	}
	virtualCoinOl, err := db.VirtualCoinOrderRelateListByOid(eg, order.OrdId, order.Pvd)
	if err != nil {
		_ = logx.Warn(err)
		return
	}
	session := eg.NewSession()
	defer session.Close()
	if err := session.Begin(); err != nil {
		_ = logx.Warn(err)
		return
	}
	//查询优惠券使用记录
	goodsType := ""
	if utils.InArr(order.Pvd, []string{md.PVD_CARD, md.PVD_OILSTATION}) {
		goodsType = order.Pvd
	}
	couponList, _ := db.CouponUserOrdByGidAscBatch(eg, order.Uid, order.ItemId, 1, goodsType, order.Pvd)
	if couponList != nil {
		//更新使用状态
		couponList.RefundState = 0
		db.CouponUserOrdByUpdate(eg, couponList, "refund_state")
	}
	now := time.Now()
	var relateFirst model.OrdListRelate
	get, _ := session.Where("oid=?", order.OrdId).Get(&relateFirst)
	mode := ""
	if get {
		mode = relateFirst.Mode
	}
	if mode == "lv_integral_release" {
		//定制处理
		bools := Deduct(session, order, dbName)
		if bools == false {
			_ = logx.Warn(err)
			_ = session.Rollback()
			return
		}
	} else {
		commRefund(eg, session, order, order.Uid, order.Pvd, order.ItemId, ol, title, dbName)
		commVirtualCoinRefund(session, order.Uid, order.Pvd, order.ItemId, virtualCoinOl, title)
		commTikTokRefund(session, order.Uid, order.Pvd, order.ItemId, tikTokOl, title)
	}
	commAreaRefund(session, order.Uid, utils.Int64ToStr(order.OrdId), md.PVD_GUIDE, order.ItemId, title)
	commStoreRefund(eg, session, utils.Int64ToStr(order.OrdId), order.Pvd)
	// 更新订单状态
	tt := utils.StrToInt(utils.AnyToString(now.Unix()))
	order.State = 4
	order.UpdateAt = tt
	order.IsRefund = 1
	order.SettleAt = 0
	_, err = db.OrderUpdateWithSession(session, order.OrdId, order)
	if err != nil {
		_ = logx.Warn(err)
		_ = session.Rollback()
		return
	}
	//if affected == 0 {
	//	_ = session.Rollback()
	//	_ = logx.Warn("ORDER SETTLE: UPDATE ORDER RETURN AFFECTED 0")
	//	return
	//}
	//判断是否消费存
	if returnMoneyType == "1" {
		err = db.WaitReturnMoneyFlowUpdateIsNeedReduce(session, order.Pvd, utils.Int64ToStr(order.OrdId), 1)
		if err != nil {
			_ = logx.Warn(err)
			_ = session.Rollback()
			return
		}
	}
	// 提交事务
	err = session.Commit()
	if err != nil {
		_ = logx.Warn(err)
		return
	}
	return
}
func NotSettleDeduct(eg *xorm.Engine, ord *model.OrdList, masterId string) bool {
	if ord.IsFastSign == 1 && ord.IsWaitReduce == 0 {
		FastReturnReduceOrder(eg, ord.Uid, utils.Int64ToStr(ord.OrdId), masterId)
	}
	sess := eg.NewSession()
	defer func() {
		_ = sess.Close()
	}()
	_ = sess.Begin()
	integralRelease, _ := db2.IntegralReleaseGetOneByParams(sess, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if integralRelease == nil {
		sess.Commit()
		return true
	}
	// 加锁 防止并发退款
	mutexKey := fmt.Sprintf("%s:all_order_refund_by_rule:%d:pvd:%s", masterId, ord.OrdId, ord.Pvd)
	available, err1 := cache.Do("SET", mutexKey, 1, "EX", 300, "NX")
	if err1 != nil {
		sess.Commit()
		return true
	}
	if available != "OK" {
		sess.Commit()
		return true
	}
	var tmp1 model.UserVirtualCoinFlow
	get1, _ := sess.Where("ord_id=? and transfer_type=?", ord.OrdId, 139).Get(&tmp1)
	if get1 {
		sess.Commit()
		return true
	}
	//没结算直接扣回红积分
	var tmp model.UserVirtualCoinFlow
	get, _ := sess.Where("ord_id=?", ord.OrdId).Get(&tmp)
	if get {
		err := rule.DealUserCoinForIntegralRelease(sess, md2.DealUserCoinReq{
			Kind:         "sub",
			Mid:          masterId,
			Title:        md2.IntegralReleaseRedCoinGrantByRefundForUserVirtualCoinFlow,
			TransferType: md2.IntegralReleaseRedCoinGrantByOrdRefundTypeForUserVirtualCoinFlow,
			OrdId:        utils.Int64ToStr(ord.OrdId),
			CoinId:       tmp.CoinId,
			Uid:          tmp.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(tmp.Amout),
		})
		if err != nil {
			sess.Rollback()
			return false
		}
	}
	sess.Commit()
	return true
}
func Deduct(sess *xorm.Session, ord *model.OrdList, masterId string) bool {
	integralRelease, _ := db2.IntegralReleaseGetOneByParams(sess, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if integralRelease == nil {
		return true
	}
	rule.InitForIntegralRelease(cfg.RedisAddr)
	// 加锁 防止并发退款
	mutexKey := fmt.Sprintf("%s:all_order_refund_by_rule:%d:pvd:%s", masterId, ord.OrdId, ord.Pvd)
	available, err1 := cache.Do("SET", mutexKey, 1, "EX", 300, "NX")
	if err1 != nil {
		return true
	}
	if available != "OK" {
		return true
	}
	//定制
	/***
	已结算
	1.读取释放表扣除对应的红积分 绿积分
	2.扣除直推的佣金
	3.读取佣金释放表扣除间推佣金
	*/

	integralOrd, _ := db2.IntegralReleaseUserWithOrdGetOneByParams(sess, map[string]interface{}{
		"key":   "ord_id",
		"value": ord.OrdId,
	})

	if integralOrd != nil {
		if integralOrd.State == 2 {
			return true
		}
		integralOrd.State = 2
		sess.Where("id=?", integralOrd.Id).Cols("state").Update(integralOrd)
		var err error
		if zhios_order_relate_utils.StrToFloat64(integralOrd.TotalRedCoinNum)-zhios_order_relate_utils.StrToFloat64(integralOrd.ReleasedGreenCoinNum) > 0 {
			err = rule.DealUserCoinForIntegralRelease(sess, md2.DealUserCoinReq{
				Kind:         "sub",
				Mid:          masterId,
				Title:        md2.IntegralReleaseRedCoinGrantByRefundForUserVirtualCoinFlow,
				TransferType: md2.IntegralReleaseRedCoinGrantByOrdRefundTypeForUserVirtualCoinFlow,
				OrdId:        utils.Int64ToStr(ord.OrdId),
				CoinId:       integralRelease.Coin1,
				Uid:          integralOrd.Uid,
				Amount:       utils.FloatFormat(zhios_order_relate_utils.StrToFloat64(integralOrd.TotalRedCoinNum)-zhios_order_relate_utils.StrToFloat64(integralOrd.ReleasedGreenCoinNum), 5),
			})
			if err != nil {
				return false
			}
		}
		if zhios_order_relate_utils.StrToFloat64(integralOrd.ReleasedGreenCoinNum) > 0 {
			err = rule.DealUserCoinForIntegralRelease(sess, md2.DealUserCoinReq{
				Kind:         "sub",
				Mid:          masterId,
				Title:        md2.IntegralReleaseGreenCoinGrantByRefundForUserVirtualCoinFlow,
				TransferType: md2.IntegralReleaseGreenCoinGrantByOrdRefundTypeForUserVirtualCoinFlow,
				OrdId:        utils.Int64ToStr(ord.OrdId),
				CoinId:       integralRelease.Coin2,
				Uid:          integralOrd.Uid,
				Amount:       zhios_order_relate_utils.StrToFloat64(integralOrd.ReleasedGreenCoinNum),
			})
			if err != nil {
				return false
			}
		}

	}
	// 2.扣除直推的佣金
	var listRelates []model.OrdListRelate
	err := sess.Table("ord_list_relate").
		Where("oid = ?", ord.OrdId).
		Find(&listRelates)
	if err != nil {
		return false
	}
	now := time.Now()
	for _, item := range listRelates {
		if item.Amount > 0 && item.Level == 2 {
			req := md2.DealIntegralReleaseInterpositionUserAmountReq{
				Kind:   "sub",
				Mid:    masterId,
				OrdId:  utils.Int64ToStr(ord.OrdId),
				CoinId: 0,
				Uid:    item.Uid,
				Amount: item.Amount,
			}
			err = DealIntegralReleaseInterpositionUserAmountRefund(sess, req)
			if err != nil {
				return false
			}
		}
		if item.Amount > 0 && item.Level != 2 {
			isSuccess, _ := refundSettleAmountToUserProfile(sess, item, now)
			if !isSuccess {
				return false
			}
		}
	}

	return true
}

func refundSettleAmountToUserProfile(sess *xorm.Session, item model.OrdListRelate, now time.Time) (bool, error) {
	userProfileItem, err := db.UserProfileFindByIdWithSession(sess, item.Uid)
	if err != nil {
		return false, err
	}
	amountItem := decimal.NewFromFloat(item.Amount)
	beforeAmountItem, err := decimal.NewFromString(userProfileItem.FinValid)
	if err != nil {
		return false, err
	}
	afterAmountItem := beforeAmountItem.Sub(amountItem)
	userProfileItem.FinValid = afterAmountItem.String()
	// 总收益计算
	userProfileItem.FinTotal = utils.StrToFloat32(decimal.NewFromFloat32(userProfileItem.FinTotal).Sub(amountItem).String())
	// 更新用户余额
	_, err = db.UserProfileUpdateWithSession(sess, item.Uid, userProfileItem, "fin_valid", "fin_total")
	if err != nil {
		return false, err
	}
	var finUserFlowItem = model.FinUserFlow{
		Uid:          item.Uid,
		Type:         1,
		Amount:       amountItem.String(),
		BeforeAmount: beforeAmountItem.String(),
		AfterAmount:  afterAmountItem.String(),
		OrdType:      item.Pvd,
		OrdId:        utils.AnyToString(item.Oid),
		OrdAction:    22,
		OrdTime:      int(now.Unix()),
		OrdDetail:    utils.AnyToString(item.Oid),
		State:        2,
		OtherId:      item.Id,
		OrdTitle:     "订单退款-分佣退还",
		CreateAt:     now.Local(),
		UpdateAt:     now.Local(),
	}
	_, err = sess.Table("fin_user_flow").Insert(&finUserFlowItem)
	if err != nil {
		return false, err
	}
	return true, nil
}

// DealIntegralReleaseInterpositionUserAmount 处理间推用户服务收益 退款扣除
func DealIntegralReleaseInterpositionUserAmountRefund(session *xorm.Session, req md2.DealIntegralReleaseInterpositionUserAmountReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、计算&&组装数据
	userAmount, err := db2.IntegralReleaseInterpositionUserAmountGetOneByParams(session, req.CoinId, map[string]interface{}{
		"key":   "uid",
		"value": req.Uid,
	})
	if err != nil && err.Error() == "not found" {
		return nil
	}
	if err != nil {
		return err
	}
	var amount float64 = 0
	if req.CoinId == 0 {
		amount = db2.GetIntegralReleaseInterpositionUserFlowSum(session, req.Uid, req.CoinId, req.OrdId)
		if amount > 0 {
			orderType := enum.FinUserFlowOrderActionString(md2.IntegralReleaseServiceRevenueOrderRefundTypeForFinUserFlow)
			var dealUserAmount = md2.DealUserAmount{
				Kind:      req.Kind,
				Mid:       req.Mid,
				Title:     md2.IntegralReleaseServiceRevenueRefundTitleForFinUserFlow,
				OrderType: orderType,
				OrdAction: md2.IntegralReleaseServiceRevenueOrderRefundTypeForFinUserFlow,
				OrdId:     req.OrdId,
				Uid:       req.Uid,
				Amount:    amount,
			}
			err = svc.DealUserAmount(session, dealUserAmount)
			if err != nil {
				return err
			}
		}
	}
	userAmount.AlreadyAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(userAmount.AlreadyAmount)-amount, 5)
	userAmount.WaiteAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(userAmount.WaiteAmount)-(req.Amount-amount), 5)
	userAmount.TotalAmount = utils.Float64ToStrByPrec(utils.StrToFloat64(userAmount.TotalAmount)-req.Amount, 5)
	updateAffected, _ := db2.IntegralReleaseInterpositionUserAmountUpdate(session, userAmount.Id, userAmount)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		return errors.New("更新 integral_release_interposition_user_flow 记录失败")
	}
	return nil
}
